# -*- coding: utf8 -*-
import sys
import re
import os
import time
import json
from imp import reload

reload(sys)
if sys.version_info < (3, 0):
    sys.setdefaultencoding('utf-8')

sys.path.append('uaDevice')
import uaDevice

if __name__ == '__main__':
    f = open('test/data.txt')
    # f = open('test/data10000.txt')
    data = f.read()

    uas = data.split('\n')

    stat = {
        'osName': [0, [], []],
        'osVersion': [0, [], []],
        'browserName': [0, [], []],
        'browserVersion': [0, [], []],
        'engineName': [0, [], []],
        'engineVersion': [0, [], []],
        'deviceType': [0, [], []],
        'deviceModel': [0, [], []],
        'deviceManufacturer': [0, [], []]
    }

    total = len(uas)
    start = time.time() * 1000
    for i in range(total):
        ua = uas[i]
        # if i + 1 != 1955:
        #     continue
        # print 'ua: ', ua, len(ua)

        s = time.time() * 1000
        info = uaDevice.parseUA(ua)
        output = '\t'.join([
            info['os']['name'],
            info['os']['version']['original'],
            info['browser']['name'],
            info['browser']['version']['original'],
            info['engine']['name'],
            info['engine']['version']['original'],
            info['device']['type'],
            info['device']['model'],
            info['device']['manufacturer']
        ])
        print(output)

        stat['osName'][2].append(info['os']['name'])
        if not info['os']['name']:
            stat['osName'][0] += 1
            stat['osName'][1].append(info['os']['name'])

        stat['osVersion'][2].append(info['os']['version']['original'])
        if not info['os']['version']['original']:
            stat['osVersion'][0] += 1
            stat['osVersion'][1].append(info['os']['version']['original'])

        stat['browserName'][2].append(info['browser']['name'])
        if not info['browser']['name']:
            stat['browserName'][0] += 1
            stat['browserName'][1].append(info['browser']['name'])

        stat['browserVersion'][2].append(info['browser']['version']['original'])
        if not info['browser']['version']['original']:
            stat['browserVersion'][0] += 1
            stat['browserVersion'][1].append(info['browser']['version']['original'])

        stat['engineName'][2].append(info['engine']['name'])
        if not info['engine']['name']:
            stat['engineName'][0] += 1
            stat['engineName'][1].append(info['engine']['name'])

        stat['engineVersion'][2].append(info['engine']['version']['original'])
        if not info['engine']['version']['original']:
            stat['engineVersion'][0] += 1
            stat['engineVersion'][1].append(info['engine']['version']['original'])

        stat['deviceType'][2].append(info['device']['type'])
        if not info['device']['type']:
            stat['deviceType'][0] += 1
            stat['deviceType'][1].append(info['device']['type'])

        stat['deviceModel'][2].append(info['device']['model'])
        if not info['device']['model']:
            stat['deviceModel'][0] += 1
            stat['deviceModel'][1].append(info['device']['model'])

        stat['deviceManufacturer'][2].append(info['device']['manufacturer'])
        if not info['device']['manufacturer']:
            stat['deviceManufacturer'][0] += 1
            stat['deviceManufacturer'][1].append(info['device']['manufacturer'])


    for key in stat.keys():
        sys.stderr.write("%s: 成功数:%s， 成功率: %s%% \n" % (key, str(total - stat[key][0]), str(float(total - stat[key][0])/total*100)))

    sys.stderr.write("Total ua number: %s ,Total time %s %sms \n" % (str(total), str(i), str(time.time() * 1000  - start)))


# -*- coding: utf8 -*-
import sys
import re
import os
import json
from imp import reload
reload(sys)
if sys.version_info < (3, 0):
    sys.setdefaultencoding('utf-8')
sys.path.append('./')

from models import *

regObj = {

}

def optimizedSearch(regStr, matchStr, flags=0):
    if regStr not in regObj:
        regObj[regStr] = re.compile(regStr, flags)

    return regObj[regStr].search(matchStr)


def optimizedSub(regStr, replaceStr, matchStr, count=0, flags=0):
    if regStr not in regObj:
        regObj[regStr] = re.compile(regStr, flags)

    return regObj[regStr].sub(replaceStr, matchStr, count=count)


class Version(object):
    def __init__(self, v={}):
        """
        初始化
        """
        self.original = v['value'] if 'value' in v else ''
        self.alias = v['alias'] if 'alias' in v else ''
        self.details = v['details'] if 'details' in v else None
        self.minor = -1
        self.type = ''
    def __str__(self):
        return self.original


class UA(object):
    def __init__(self, ua, options=None):
        """
        初始化
        """

        self.options = {
            'useFeatures': options['useFeatures'] if options else False,
            'detectCamouflage': options['detectCamouflage'] if options else True
        }

        self.browser = {
            'name': '',
            'version': Version(),
            'mode': '', # desktop mobile
            'details': 0,
            'stock': True,
            'hidden': False,
            'channel': ''
        }
        self.engine = {
            'name': '',  # Webkit Presto
            'version': Version()
        }
        self.os = {
            'name': '',
            'version': Version()
        }
        self.device = {
            'type': 'desktop',
            'identified': False,
            'manufacturer': '',
            'model': '',
        }

        self.camouflage = False
        self.features = []
        if ua:
            self.detect(ua)


    def getUaInfo(self):
        return {
            'os': {
                'name': self.os['name'],
                'version': {
                    'alias': self.os['version'].alias if self.os['version'] else '',
                    'original': self.os['version'].original if self.os['version'] else '',
                }
            },
            'browser': {
                'name': self.browser['name'],
                'mode': self.browser['mode'],
                'version': {
                    'alias': self.browser['version'].alias if self.browser['version'] else '',
                    'original': self.browser['version'].original if self.browser['version'] else '',
                }
            },
            'engine': {
                'name': self.engine['name'],
                'version': {
                    'alias': self.engine['version'].alias,
                    'original': self.engine['version'].original
                }
            },
            'device': {
                'type': self.device['type'] if self.device['identified'] else '',
                'manufacturer': self.device['manufacturer'],
                'model': self.device['model']
            }
        }

    def detect(self, ua):


        # Unix
        #

        if 'Unix' in ua:
            self.os['name'] = 'Unix'



        # FreeBSD
        #

        if 'FreeBSD' in ua:
            self.os['name'] = 'FreeBSD'



        # OpenBSD
        #

        if 'OpenBSD' in ua:
            self.os['name'] = 'OpenBSD'



        # NetBSD
        #

        if 'NetBSD' in ua:
            self.os['name'] = 'NetBSD'



        # SunOS
        #

        if 'SunOS' in ua:
            self.os['name'] = 'Solaris'



        # Linux
        #

        if 'Linux' in ua:
            self.os['name'] = 'Linux'

            if 'CentOS' in ua:
                self.os['name'] = 'CentOS'
                match = optimizedSearch(r'CentOS\/[0-9\.\-]+el([0-9_]+)', ua)
                if match:
                    self.os['version'] = Version({
                        'value': match.group(1).replace('_', '.')
                    })



            if 'Debian' in ua:
                self.os['name'] = 'Debian'


            if 'Fedora' in ua:
                self.os['name'] = 'Fedora'
                match = optimizedSearch(r'Fedora\/[0-9\.\-]+fc([0-9]+)', ua)
                if match:
                    self.os['version'] = Version({
                        'value': match.group(1)
                    })



            if 'Gentoo' in ua:
                self.os['name'] = 'Gentoo'


            if 'Kubuntu' in ua:
                self.os['name'] = 'Kubuntu'


            if 'Mandriva Linux' in ua:
                self.os['name'] = 'Mandriva'
                match = optimizedSearch(r'Mandriva Linux\/[0-9\.\-]+mdv([0-9]+)', ua)
                if match:
                    self.os['version'] = Version({
                        'value': match.group(1)
                    })



            if 'Mageia' in ua:
                self.os['name'] = 'Mageia'
                match = optimizedSearch(r'Mageia\/[0-9\.\-]+mga([0-9]+)', ua)
                if match:
                    self.os['version'] = Version({
                        'value': match.group(1)
                    })



            if 'Red Hat' in ua:
                self.os['name'] = 'Red Hat'
                match = optimizedSearch(r'Red Hat[^\/]*\/[0-9\.\-]+el([0-9_]+)', ua)
                if match:
                    self.os['version'] = Version({
                        'value': match.group(1).replace('_', '.')
                    })



            if 'Slackware' in ua:
                self.os['name'] = 'Slackware'


            if 'SUSE' in ua:
                self.os['name'] = 'SUSE'


            if 'Turbolinux' in ua:
                self.os['name'] = 'Turbolinux'


            if 'Ubuntu' in ua:
                self.os['name'] = 'Ubuntu'
                match = optimizedSearch(r'Ubuntu\/([0-9.]*)', ua)
                if match:
                    self.os['version'] = Version({
                        'value': match.group(1)
                    })



        # iOS
        #
        if optimizedSearch(r'iPhone( Simulator)?;?', ua) or 'iPad;' in ua or 'iPod;' in ua or optimizedSearch(r'iPhone\s*\d*s?[cp]?;', ua, re.I):
            self.os['name'] = 'iOS'
            self.os['version'] = Version({
                'value': '1.0'
            })
            # fix: 懒匹配，不然会包含like等
            match = optimizedSearch(r'OS (.*?) like Mac OS X', ua)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1).replace('_', '.')
                })


            if 'iPhone Simulator;' in ua:
                self.device['type'] = 'emulator'
            elif 'iPod;' in ua:
                self.device['type'] = 'media'
                self.device['manufacturer'] = 'Apple'
                self.device['model'] = 'iPod Touch'
            elif 'iPhone;' in ua or optimizedSearch(r'iPhone\s*\d*s?[cp]?;?', ua, re.I):
                self.device['type'] = 'mobile'
                self.device['manufacturer'] = 'Apple'
                self.device['model'] = 'iPhone'
            else:
                self.device['type'] = 'tablet'
                self.device['manufacturer'] = 'Apple'
                self.device['model'] = 'iPad'


            self.device['identified'] = True



        # MacOS X
        #

        elif 'Mac OS X' in ua:
            self.os['name'] = 'Mac OS X'
            match = optimizedSearch(r'Mac OS X (10[0-9\._]*)', ua)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1).replace('_', '.')
                })




        # Windows
        #

        if 'Windows' in ua:
            self.os['name'] = 'Windows'
            match = optimizedSearch(r'Windows NT ([0-9]\.[0-9])', ua)
            if match:
                self.os['version'] = parseVersion(match.group(1))
                tmpMap = {
                    '6.2': '8',
                    '6.1': '7',
                    '6.0': 'Vista',
                    '5.2': 'Server 2003',
                    '5.1': 'XP',
                    '5.0': '2000'
                }
                if match.group(1) in tmpMap:
                    self.os['version'] = Version({
                        'value': match.group(1),
                        'alias': tmpMap[match.group(1)]
                    })
                else:
                    self.os['version'] = Version({
                        'value': match.group(1),
                        'alias': 'NT ' + str(self.os['version'])
                    })



            if 'Windows 95' in ua or 'Win95' in ua or 'Win 9x 4.00' in ua:
                self.os['version'] = Version({
                    'value': '4.0',
                    'alias': '95'
                })


            if 'Windows 98' in ua or 'Win98' in ua or 'Win 9x 4.10' in ua:
                self.os['version'] = Version({
                    'value': '4.1',
                    'alias': '98'
                })


            if 'Windows ME' in ua or 'WinME' in ua or 'Win 9x 4.90' in ua:
                self.os['version'] = Version({
                    'value': '4.9',
                    'alias': 'ME'
                })


            if 'Windows XP' in ua or 'WinXP' in ua:
                self.os['name'] = Version({
                    'value': '5.1',
                    'alias': 'XP'
                })


            if 'WP7' in ua:
                self.os['name'] = 'Windows Phone'
                self.os['version'] = Version({
                    'value': '7.0',
                    'details': 2
                })
                self.device['type'] = 'mobile'
                self.browser['mode'] = 'desktop'


            if 'Windows CE' in ua or 'WinCE' in ua or 'WindowsCE' in ua:
                if ' IEMobile' in ua:
                    self.os['name'] = 'Windows Mobile'

                    if ' IEMobile 8' in ua:
                        self.os['version'] = Version({
                            'value': '6.5',
                            'details': 2
                        })


                    if ' IEMobile 7' in ua:
                        self.os['version'] = Version({
                            'value': '6.1',
                            'details': 2
                        })


                    if ' IEMobile 6' in ua:
                        self.os['version'] = Version({
                            'value': '6.0',
                            'details': 2
                        })

                else:
                    self.os['name'] = 'Windows CE'
                    match = optimizedSearch(r'WindowsCEOS\/([0-9.]*)', ua)
                    if match:
                        self.os['version'] = Version({
                            'value': match.group(1),
                            'details': 2
                        })

                    match = optimizedSearch(r'Windows CE ([0-9.]*)', ua)
                    if match:
                        self.os['version'] = Version({
                            'value': match.group(1),
                            'details': 2
                        })



                self.device['type'] = 'mobile'


            if 'Windows Mobile' in ua:
                self.os['name'] = 'Windows Mobile'
                self.device['type'] = 'mobile'

            match = optimizedSearch(r'WindowsMobile\/([0-9.]*)', ua)
            if match:
                self.os['name'] = 'Windows Mobile'
                self.os['version'] = Version({
                    'value': match.group(1),
                    'details': 2
                })
                self.device['type'] = 'mobile'

            match = optimizedSearch(r'Windows Phone [0-9]', ua)
            if match:
                self.os['name'] = 'Windows Mobile'
                self.os['version'] = Version({
                    'value': optimizedSearch(r'Windows Phone ([0-9.]*)', ua).group(1),
                    'details': 2
                })
                self.device['type'] = 'mobile'


            if 'Windows Phone OS' in ua:
                self.os['name'] = 'Windows Phone'

                # fix
                ver = optimizedSearch(r'Windows Phone OS ([0-9.]*)', ua).group(1)
                self.os['version'] = Version({
                    'value': ver,
                    'details': 2
                })
                ver = ver or '0'
                ver = float('.'.join(ver.split('.')[:2])) # 取两位
                if ver < 7:
                    self.os['name'] = 'Windows Mobile'

                match = optimizedSearch(r'IEMobile\/[^;]+; ([^;]+); ([^;]+)[;|\)]', ua)
                if match:
                    self.device['manufacturer'] = match.group(1)
                    self.device['model'] = match.group(2)


                self.device['type'] = 'mobile'

                manufacturer = self.device['manufacturer']
                model = cleanupModel(self.device['model'])

                if manufacturer in WINDOWS_PHONE_MODELS and model in WINDOWS_PHONE_MODELS[manufacturer]:
                    self.device['manufacturer'] = WINDOWS_PHONE_MODELS[manufacturer][model][0]
                    self.device['model'] = WINDOWS_PHONE_MODELS[manufacturer][model][1]
                    self.device['identified'] = True


                if manufacturer is 'Microsoft' and model is 'XDeviceEmulator':
                    self.device['manufacturer'] = None
                    self.device['model'] = None
                    self.device['type'] = 'emulator'
                    self.device['identified'] = True





        # Android
        #
        if 'Android' in ua:
            self.os['name'] = 'Android'
            self.os['version'] = None

            match = optimizedSearch(r'Android(?: )?(?:AllPhone_|CyanogenMod_)?(?:\/)?v?([0-9.]+)', ua.replace('-update', '.'))
            if match:
                self.os['version'] = Version({
                    'value': match.group(1),
                    'details': 3
                })


            if 'Android Eclair' in ua:
                self.os['version'] = Version({
                    'value': '2.0',
                    'details': 3
                })


            self.device['type'] = 'mobile'

            # fix: 原js里，可能是个bug，version不是数字，是对象
            if self.os['version']:
                ver = '0'
                if self.os['version'].original and self.os['version'].original != '.':
                    ver = self.os['version'].original
                ver = float('.'.join(ver.split('.')[:2])) # 取两位
                if ver >= 3:
                    self.device['type'] = 'tablet'
                if ver >= 4 and 'Mobile' in ua:
                    self.device['type'] = 'mobile'

                # 比原js多的逻辑
                if 'ZTEQ801' in ua:
                    self.device['type'] = 'mobile'


            tmpArr = [
                'Eclair; (?:[a-zA-Z][a-zA-Z](?:[-_][a-zA-Z][a-zA-Z])?) Build\/([^\/]*)\/',
                '; ([^;]*[^;\s])\s+Build',
                '[a-zA-Z][a-zA-Z](?:[-_][a-zA-Z][a-zA-Z])?; ([^;]*[^;\s]);\s+Build',
                '\(([^;]+);U;Android\/[^;]+;[0-9]+\*[0-9]+;CTC\/2.0\)',
                ';\s?([^;]+);\s?[0-9]+\*[0-9]+;\s?CTC\/2.0',
                'zh-cn;\s*(.*?)(\/|build)',
                'Android [^;]+; (?:[a-zA-Z][a-zA-Z](?:[-_][a-zA-Z][a-zA-Z])?; )?([^)]+)\)',
                '^(.+?)\/\S+'
            ]
            for regstr in tmpArr:
                if regstr ==  tmpArr[-1]:
                    match = optimizedSearch(regstr, ua, re.I)
                else:
                    match = optimizedSearch(regstr, ua)

                if match:
                    if regstr == tmpArr[-2]:
                        if not optimizedSearch(r'[a-zA-Z][a-zA-Z](?:[-_][a-zA-Z][a-zA-Z])?', ua):
                            self.device['model'] = match.group(1)
                    else:
                        self.device['model'] = match.group(1)
                    break
            # origin
            # if match:

            # elif match = //.exec(ua):
            #     self.device['model'] = match.group(1)
            # elif match = //.exec(ua):
            #     self.device['model'] = match.group(1)
            # elif match = //.exec(ua):
            #     self.device['model'] = match.group(1)
            # elif match = //.exec(ua):
            #     self.device['model'] = match.group(1)
            # elif match = //i.exec(ua):
            #     self.device['model'] = match.group(1)
            # elif match = //.exec(ua):
            #     if not ua.match(/[a-zA-Z][a-zA-Z](?:[-_][a-zA-Z][a-zA-Z])?/):
            #         self.device['model'] = match.group(1)

            # elif match = /^(.+?)\/\S+/i.exec(ua):
            #     self.device['model'] = match.group(1)



            """Sometimes we get a model name that starts with Android, in that case it is a mismatch and we should ignore it"""
            if self.device['model'] and self.device['model'][0:7] is 'Android':
                self.device['model'] = None


            if self.device['model']:
                model = cleanupModel(self.device['model'])

                if model in ANDROID_MODELS:
                    self.device['manufacturer'] = ANDROID_MODELS[model][0]
                    self.device['model'] = ANDROID_MODELS[model][1]
                    if len(ANDROID_MODELS[model]) > 2:
                        self.device['type'] = ANDROID_MODELS[model][2]
                    self.device['identified'] = True


                if model == 'Emulator ' or model == 'x86 Emulator' or model == 'x86 VirtualBox' or model == 'vm':
                    self.device['manufacturer'] = None
                    self.device['model'] = None
                    self.device['type'] = 'emulator'
                    self.device['identified'] = True



            if 'HP eStation' in ua:
                self.device['manufacturer'] = 'HP'
                self.device['model'] = 'eStation'
                self.device['type'] = 'tablet'
                self.device['identified'] = True

            match = optimizedSearch(r'Pre\/1.0', ua)
            if match:
                self.device['manufacturer'] = 'Palm'
                self.device['model'] = 'Pre'
                self.device['identified'] = True

            match = optimizedSearch(r'Pre\/1.1', ua)
            if match:
                self.device['manufacturer'] = 'Palm'
                self.device['model'] = 'Pre Plus'
                self.device['identified'] = True

            match = optimizedSearch(r'Pre\/1.2', ua)
            if match:
                self.device['manufacturer'] = 'Palm'
                self.device['model'] = 'Pre 2'
                self.device['identified'] = True

            match = optimizedSearch(r'Pre\/3.0', ua)
            if match:
                self.device['manufacturer'] = 'HP'
                self.device['model'] = 'Pre 3'
                self.device['identified'] = True

            match = optimizedSearch(r'Pixi\/1.0', ua)
            if match:
                self.device['manufacturer'] = 'Palm'
                self.device['model'] = 'Pixi'
                self.device['identified'] = True

            match = optimizedSearch(r'Pixi\/1.1', ua)
            if match:
                self.device['manufacturer'] = 'Palm'
                self.device['model'] = 'Pixi Plus'
                self.device['identified'] = True

            match = optimizedSearch(r'P160UN?A?\/1.0', ua)
            if match:
                self.device['manufacturer'] = 'HP'
                self.device['model'] = 'Veer'
                self.device['identified'] = True




        # Google TV
        #

        if 'GoogleTV' in ua:
            self.os['name'] = 'Google TV'
            match = optimizedSearch(r'Chrome/5.', ua)
            if match:
                self.os['version'] = Version({
                    'value': '1'
                })

            match = optimizedSearch(r'Chrome/11.', ua)
            if match:
                self.os['version'] = Version({
                    'value': '2'
                })


            self.device['type'] = 'television'



        # WoPhone
        #

        if 'WoPhone' in ua:
            self.os['name'] = 'WoPhone'
            match = optimizedSearch(r'WoPhone\/([0-9\.]*)', ua)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1)
                })


            self.device['type'] = 'mobile'



        # BlackBerry
        #

        if 'BlackBerry' in ua:
            self.os['name'] = 'BlackBerry OS'

            if 'Opera' not in ua:
                match = optimizedSearch(r'BlackBerry([0-9]*)\/([0-9.]*)', ua)
                if match:
                    self.device['model'] = match.group(1)
                    self.os['version'] = Version({
                        'value': match.group(2),
                        'details': 2
                    })

                match = optimizedSearch(r'; BlackBerry ([0-9]*);', ua)
                if match:
                    self.device['model'] = match.group(1)

                match = optimizedSearch(r'Version\/([0-9.]*)', ua)
                if match:
                    self.os['version'] = Version({
                        'value': match.group(1),
                        'details': 2
                    })

                # fix
                ver = '0'
                if self.os['version']:
                    ver = self.os['version'].original
                ver = float('.'.join(ver.split('.')[:2])) # 只要一个dot，否则float报错
                if ver >= 10:
                    self.os['name'] = 'BlackBerry'


                if self.device['model']:
                    if self.device['model'] in BLACKBERRY_MODELS:
                        self.device['model'] = 'BlackBerry ' + BLACKBERRY_MODELS[self.device['model']] + ' ' + self.device['model']
                    else:
                        self.device['model'] = 'BlackBerry ' + self.device['model']

                else:
                    self.device['model'] = 'BlackBerry'

            else:
                self.device['model'] = 'BlackBerry'


            self.device['manufacturer'] = 'RIM'
            self.device['type'] = 'mobile'
            self.device['identified'] = True



        # BlackBerry PlayBook
        #

        if 'RIM Tablet OS' in ua:
            self.os['name'] = 'BlackBerry Tablet OS'

            self.os['version'] = Version({
                'value': optimizedSearch(r'RIM Tablet OS ([0-9.]*)', ua).groups(1),
                'details': 2
            })

            self.device['manufacturer'] = 'RIM'
            self.device['model'] = 'BlackBerry PlayBook'
            self.device['type'] = 'tablet'
            self.device['identified'] = True
        elif 'PlayBook' in ua:
            match = optimizedSearch(r'Version\/(10[0-9.]*)', ua)
            if match:
                self.os['name'] = 'BlackBerry'
                self.os['version'] = Version({
                    'value': match.group(1),
                    'details': 2
                })

                self.device['manufacturer'] = 'RIM'
                self.device['model'] = 'BlackBerry PlayBook'
                self.device['type'] = 'tablet'
                self.device['identified'] = True





        # WebOS
        #

        if optimizedSearch(r'(?:web|hpw)OS', ua):
            self.os['name'] = 'webOS'
            self.os['version'] = Version({
                'value': optimizedSearch(r'(?:web|hpw)OS\/([0-9.]*)', ua).groups(1),
            })

            if 'tablet' in ua:
                self.device['type'] = 'tablet'
            else:
                self.device['type'] = 'mobile'

            self.device['manufacturer'] = 'HP' if 'hpwOS' in ua else 'Palm'
            tmpMap = {
                'Pre\/1.0': 'Pre',
                'Pre\/1.1': 'Pre Plus',
                'Pre\/1.2': 'Pre2',
                'Pre\/3.0': 'Pre3',
                'Pixi\/1.0': 'Pixi',
                'Pixi\/1.1': 'Pixi Plus',
                'P160UN?A?\/1.0': 'Veer',
                'TouchPad\/1.0': 'TouchPad'
            }
            for item in tmpMap.keys():
                if optimizedSearch(item, ua):
                    self.device['model'] = tmpMap[item]
            # origin
            # if ua.match('Pre\/1.0')) self.device['model'] = 'Pre'
            # if ua.match('Pre\/1.1')) self.device['model'] = 'Pre Plus'
            # if ua.match('Pre\/1.2')) self.device['model'] = 'Pre2'
            # if ua.match('Pre\/3.0')) self.device['model'] = 'Pre3'
            # if ua.match('Pixi\/1.0')) self.device['model'] = 'Pixi'
            # if ua.match('Pixi\/1.1')) self.device['model'] = 'Pixi Plus'
            # if ua.match('P160UN?A?\/1.0')) self.device['model'] = 'Veer'
            # if ua.match('TouchPad\/1.0')) self.device['model'] = 'TouchPad'

            if optimizedSearch(r'Emulator\/', ua) or optimizedSearch(r'Desktop\/', ua):
                self.device['type'] = 'emulator'
                self.device['manufacturer'] = None
                self.device['model'] = None


            self.device['identified'] = True



        # S60
        #
        # fix S60可能是联想手机: Lenovo S60-t/V2.0 Linux/3.4.5 Android/4.4 Release/10.08.2014 Browser/AppleWebKit537.36 Chrome/30.0.0.0 Mobile Safari/537.36; 124
        if 'Symbian' in ua or optimizedSearch(r'Series[ ]?60', ua) or ('S60' in ua and 'Android' not in ua):
            self.os['name'] = 'Series60'

            if optimizedSearch(r'SymbianOS/9.1', ua) and not optimizedSearch(r'Series60', ua):
                self.os['version'] = Version({
                    'value': '3.0'
                })

            match = optimizedSearch(r'Series60\/([0-9.]*)', ua)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1)
                })

            match = optimizedSearch(r'Nokia([^\/;]+)[\/|;]', ua)
            if match:
                if match.group(1) is not 'Browser':
                    self.device['manufacturer'] = 'Nokia'
                    self.device['model'] = match.group(1)
                    self.device['identified'] = True


            match = optimizedSearch(r'Vertu([^\/;]+)[\/|;]', ua)
            if match:
                self.device['manufacturer'] = 'Vertu'
                self.device['model'] = match.group(1)
                self.device['identified'] = True

            match = optimizedSearch(r'Symbian; U; ([^;]+); [a-z][a-z]\-[a-z][a-z]', ua, re.I)
            if match:
                self.device['manufacturer'] = 'Nokia'
                self.device['model'] = match.group(1)
                self.device['identified'] = True

            match = optimizedSearch(r'Samsung\/([^;]*);', ua)
            if match:
                self.device['manufacturer'] = STRINGS_SAMSUNG
                self.device['model'] = match.group(1)
                self.device['identified'] = True


            self.device['type'] = 'mobile'



        # S40
        #

        if 'Series40' in ua:
            self.os['name'] = 'Series40'
            match = optimizedSearch(r'Nokia([^\/]+)\/', ua)
            if match:
                self.device['manufacturer'] = 'Nokia'
                self.device['model'] = match.group(1)
                self.device['identified'] = True


            self.device['type'] = 'mobile'



        # MeeGo
        #

        if 'MeeGo' in ua:
            self.os['name'] = 'MeeGo'
            self.device['type'] = 'mobile'
            match = optimizedSearch(r'Nokia([^\)]+)\)', ua)
            if match:
                self.device['manufacturer'] = 'Nokia'
                self.device['model'] = match.group(1)
                self.device['identified'] = True




        # Maemo
        #

        if 'Maemo' in ua:
            self.os['name'] = 'Maemo'
            self.device['type'] = 'mobile'
            match = optimizedSearch(r'(N[0-9]+)', ua)
            if match:
                self.device['manufacturer'] = 'Nokia'
                self.device['model'] = match.group(1)
                self.device['identified'] = True




        # Tizen
        #

        if 'Tizen' in ua:
            self.os['name'] = 'Tizen'
            match = optimizedSearch(r'Tizen[\/ ]([0-9.]*)', ua)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1)
                })


            self.device['type'] = 'mobile'
            match = optimizedSearch(r'\(([^;]+); ([^\/]+)\/', ua)
            if match:
                if match.group(1) is not 'Linux':
                    self.device['manufacturer'] = match.group(1)
                    self.device['model'] = match.group(2)

                    if self.device['manufacturer'] in TIZEN_MODELS and self.device['model'] in TIZEN_MODELS[self.device['manufacturer']]:
                        manufacturer = self.device['manufacturer']
                        model = cleanupModel(self.device['model'])

                        self.device['manufacturer'] = TIZEN_MODELS[manufacturer][model][0]
                        self.device['model'] = TIZEN_MODELS[manufacturer][model][1]
                        self.device['identified'] = True






        # Bada
        #
        if optimizedSearch(r'[b|B]ada', ua):
            self.os['name'] = 'Bada'
            match = optimizedSearch(r'[b|B]ada\/([0-9.]*)', ua)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1)
                })


            self.device['type'] = 'mobile'
            match = optimizedSearch(r'\(([^;]+); ([^\/]+)\/', ua)
            if match:
                self.device['manufacturer'] = match.group(1)
                self.device['model'] = cleanupModel(match.group(2))

            if self.device['manufacturer'] in BADA_MODELS and self.device['model'] in BADA_MODELS[self.device['manufacturer']]:
                manufacturer = self.device['manufacturer']
                model = cleanupModel(self.device['model'])

                self.device['manufacturer'] = BADA_MODELS[manufacturer][model][0]
                self.device['model'] = BADA_MODELS[manufacturer][model][1]
                self.device['identified'] = True




        # Brew
        #

        if optimizedSearch(r'BREW', ua, re.I) or optimizedSearch(r'BMP; U', ua):
            self.os['name'] = 'Brew'
            self.device['type'] = 'mobile'
            match = optimizedSearch(r'BREW; U; ([0-9.]*)', ua, re.I)
            match1 = optimizedSearch(r';BREW\/([0-9.]*)', ua, re.I)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1)
                })
            elif match1:
                self.os['version'] = Version({
                    'value': match1.group(1)
                })


            match = optimizedSearch(r'\(([^;]+);U;REX\/[^;]+;BREW\/[^;]+;(?:.*;)?[0-9]+\*[0-9]+;CTC\/2.0\)', ua)
            if match:
                self.device['model'] = match.group(1)


            if self.device['model']:
                model = cleanupModel(self.device['model'])

                if model in BREW_MODELS:
                    self.device['manufacturer'] = BREW_MODELS[model][0]
                    self.device['model'] = BREW_MODELS[model][1]
                    self.device['identified'] = True





        # MTK
        #

        if optimizedSearch(r'\(MTK;', ua):
            self.os['name'] = 'MTK'
            self.device['type'] = 'mobile'



        # CrOS
        #

        if 'CrOS' in ua:
            self.os['name'] = 'Chrome OS'
            self.device['type'] = 'desktop'



        # Joli OS
        #

        if 'Joli OS' in ua:
            self.os['name'] = 'Joli OS'
            self.device['type'] = 'desktop'
            match = optimizedSearch(r'Joli OS\/([0-9.]*)', ua, re.I)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1)
                })




        # Haiku
        #

        if 'Haiku' in ua:
            self.os['name'] = 'Haiku'
            self.device['type'] = 'desktop'



        # QNX
        #

        if 'QNX' in ua:
            self.os['name'] = 'QNX'
            self.device['type'] = 'mobile'



        # OS/2 Warp
        #

        if optimizedSearch(r'OS\/2; Warp', ua):
            self.os['name'] = 'OS/2 Warp'
            self.device['type'] = 'desktop'
            match = optimizedSearch(r'OS\/2; Warp ([0-9.]*)', ua, re.I)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1)
                })




        # Grid OS
        #

        if 'Grid OS' in ua:
            self.os['name'] = 'Grid OS'
            self.device['type'] = 'tablet'
            match = optimizedSearch(r'Grid OS ([0-9.]*)', ua, re.I)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1)
                })




        # AmigaOS
        #

        if optimizedSearch(r'AmigaOS', ua, re.I):
            self.os['name'] = 'AmigaOS'
            self.device['type'] = 'desktop'
            match = optimizedSearch(r'AmigaOS ([0-9.]*)', ua, re.I)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1)
                })





        # MorphOS
        #

        if optimizedSearch(r'MorphOS', ua, re.I):
            self.os['name'] = 'MorphOS'
            self.device['type'] = 'desktop'
            match = optimizedSearch(r'MorphOS ([0-9.]*)', ua, re.I)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1)
                })





        # Kindle
        #

        if 'Kindle' in ua and 'Fire' not in ua:
            self.os['name'] = ''

            self.device['manufacturer'] = 'Amazon'
            self.device['model'] = 'Kindle'
            self.device['type'] = 'ereader'

            if optimizedSearch(r'Kindle\/2.0', ua):
                self.device['model'] = 'Kindle 2'
            if optimizedSearch(r'Kindle\/3.0', ua):
                self.device['model'] = 'Kindle 3 or later'

            self.device['identified'] = True



        # NOOK
        #

        if 'nook browser' in ua:
            self.os['name'] = 'Android'

            self.device['manufacturer'] = 'Barnes & Noble'
            self.device['model'] = 'NOOK'
            self.device['type'] = 'ereader'
            self.device['identified'] = True



        # Bookeen
        #

        if optimizedSearch(r'bookeen\/cybook', ua):
            self.os['name'] = ''

            self.device['manufacturer'] = 'Bookeen'
            self.device['model'] = 'Cybook'
            self.device['type'] = 'ereader'

            if 'Orizon' in ua:
                self.device['model'] = 'Cybook Orizon'


            self.device['identified'] = True



        # Sony Reader
        #

        if 'EBRD1101' in ua:
            self.os['name'] = ''

            self.device['manufacturer'] = 'Sony'
            self.device['model'] = 'Reader'
            self.device['type'] = 'ereader'
            self.device['identified'] = True



        # iRiver
        #

        if 'Iriver ;' in ua:
            self.os['name'] = ''

            self.device['manufacturer'] = 'iRiver'
            self.device['model'] = 'Story'
            self.device['type'] = 'ereader'

            if 'EB07' in ua:
                self.device['model'] = 'Story HD EB07'


            self.device['identified'] = True



        # Nintendo
        #
        # Opera/9.30 (Nintendo Wii; U; ; 3642; en)
        # Opera/9.30 (Nintendo Wii; U; ; 2047-7; en)
        # Opera/9.50 (Nintendo DSi; Opera/507; U; en-US)
        # Mozilla/5.0 (Nintendo 3DS; U; ; en) Version/1.7455.US
        # Mozilla/5.0 (Nintendo 3DS; U; ; en) Version/1.7455.EU
        #

        if 'Nintendo Wii' in ua:
            self.os['name'] = ''

            self.device['manufacturer'] = 'Nintendo'
            self.device['model'] = 'Wii'
            self.device['type'] = 'gaming'
            self.device['identified'] = True


        if 'Nintendo DSi' in ua:
            self.os['name'] = ''

            self.device['manufacturer'] = 'Nintendo'
            self.device['model'] = 'DSi'
            self.device['type'] = 'gaming'
            self.device['identified'] = True


        if 'Nintendo 3DS' in ua:
            self.os['name'] = ''

            self.device['manufacturer'] = 'Nintendo'
            self.device['model'] = '3DS'
            self.device['type'] = 'gaming'
            match = optimizedSearch(r'Version\/([0-9.]*)', ua)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1)
                })


            self.device['identified'] = True



        if 'PlayStation Portable' in ua:
            self.os['name'] = ''

            self.device['manufacturer'] = 'Sony'
            self.device['model'] = 'Playstation Portable'
            self.device['type'] = 'gaming'
            self.device['identified'] = True


        if 'PlayStation Vita' in ua:
            self.os['name'] = ''
            match = optimizedSearch(r'PlayStation Vita ([0-9.]*)', ua)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1)
                })


            self.device['manufacturer'] = 'Sony'
            self.device['model'] = 'PlayStation Vita'
            self.device['type'] = 'gaming'
            self.device['identified'] = True


        if optimizedSearch(r'PlayStation 3', ua, re.I):
            self.os['name'] = ''
            match = optimizedSearch(r'PLAYSTATION 3;? ([0-9.]*)', ua)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1)
                })


            self.device['manufacturer'] = 'Sony'
            self.device['model'] = 'Playstation 3'
            self.device['type'] = 'gaming'
            self.device['identified'] = True



        # Panasonic Smart Viera
        #
        # Mozilla/5.0 (FreeBSD; U; Viera; ja-JP) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.2.4 Chrome/14.0.835.202 Safari/535.1
        #

        if 'Viera' in ua:
            self.os['name'] = ''
            self.device['manufacturer'] = 'Panasonic'
            self.device['model'] = 'Smart Viera'
            self.device['type'] = 'television'
            self.device['identified'] = True




        # Sharp AQUOS TV
        #
        # Mozilla/5.0 (DTV) AppleWebKit/531.2  (KHTML, like Gecko) AQUOSBrowser/1.0 (US00DTV;V;0001;0001)
        # Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.4 AQUOSBrowser/1.0 (CH00DTV;V;0001;0001)
        # Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10 AQUOS-AS/1.0 LC-40LE835X
        #

        if 'AQUOSBrowser' in ua or 'AQUOS-AS' in ua:
            self.os['name'] = ''
            self.device['manufacturer'] = STRINGS_SHARP
            self.device['model'] = 'Aquos TV'
            self.device['type'] = 'television'
            self.device['identified'] = True




        # Samsung Smart TV
        #
        # Mozilla/5.0 (SmartHub; SMART-TV; U; Linux/SmartTV; Maple2012) AppleWebKit/534.7 (KHTML, like Gecko) SmartTV Safari/534.7
        # Mozilla/5.0 (SmartHub; SMART-TV; U; Linux/SmartTV) AppleWebKit/531.2+ (KHTML, like Gecko) WebBrowser/1.0 SmartTV Safari/531.2+
        #

        if 'SMART-TV' in ua:
            self.os['name'] = ''
            self.device['manufacturer'] = STRINGS_SAMSUNG
            self.device['model'] = 'Smart TV'
            self.device['type'] = 'television'
            self.device['identified'] = True
            match = optimizedSearch(r'Maple([0-9]*)', ua)
            if match:
                self.device['model'] += ' ' + match.group(1)




        # Sony Internet TV
        #
        # Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-46EX640; CC/USA; en) Presto/2.8.115 Version/11.10
        # Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-40EX640; CC/USA; en) Presto/2.10.250 Version/11.60
        # Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) N/A; CC/USA; en) Presto/2.8.115 Version/11.10
        # Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) ; CC/JPN; en) Presto/2.9.167 Version/11.50
        # Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) AZ2CVT2; CC/CAN; en) Presto/2.7.61 Version/11.00
        # Opera/9.80 (Linux armv6l; Opera TV Store/4207; U; (SonyBDP/BDV11); en) Presto/2.9.167 Version/11.50
        # Opera/9.80 (Linux armv6l ; U; (SonyBDP/BDV11); en) Presto/2.6.33 Version/10.60
        # Opera/9.80 (Linux armv6l; U; (SonyBDP/BDV11); en) Presto/2.8.115 Version/11.10
        #

        if optimizedSearch(r'SonyDTV|SonyBDP|SonyCEBrowser', ua):
            self.os['name'] = ''
            self.device['manufacturer'] = 'Sony'
            self.device['model'] = 'Internet TV'
            self.device['type'] = 'television'
            self.device['identified'] = True



        # Philips Net TV
        #
        # Opera/9.70 (Linux armv6l ; U; CE-HTML/1.0 NETTV/2.0.2; en) Presto/2.2.1
        # Opera/9.80 (Linux armv6l ; U; CE-HTML/1.0 NETTV/3.0.1;; en) Presto/2.6.33 Version/10.60
        # Opera/9.80 (Linux mips; U; CE-HTML/1.0 NETTV/3.0.1; PHILIPS-AVM-2012; en) Presto/2.9.167 Version/11.50
        # Opera/9.80 (Linux mips ; U; HbbTV/1.1.1 (; Philips; ; ; ; ) CE-HTML/1.0 NETTV/3.1.0; en) Presto/2.6.33 Version/10.70
        # Opera/9.80 (Linux i686; U; HbbTV/1.1.1 (; Philips; ; ; ; ) CE-HTML/1.0 NETTV/3.1.0; en) Presto/2.9.167 Version/11.50
        #

        if optimizedSearch(r'NETTV\/', ua):
            self.os['name'] = ''
            self.device['manufacturer'] = 'Philips'
            self.device['model'] = 'Net TV'
            self.device['type'] = 'television'
            self.device['identified'] = True



        # LG NetCast TV
        #
        # Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; GLOBAL-PLAT4; 03.09.22; 0x00000001;); LG NetCast.TV-2012
        # Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+SCREEN+TUNER; LGE; GLOBAL-PLAT4; 01.00.00; 0x00000001;); LG NetCast.TV-2012
        # Mozilla/5.0 (DirectFB; U; Linux armv6l; en) AppleWebKit/531.2  (KHTML, like Gecko) Safari/531.2  LG Browser/4.1.4( BDP; LGE; Media/BD660; 6970; abc;); LG NetCast.Media-2011
        # Mozilla/5.0 (DirectFB; U; Linux 7631; en) AppleWebKit/531.2  (KHTML, like Gecko) Safari/531.2  LG Browser/4.1.4( NO_NUM; LGE; Media/SP520; ST.3.97.409.F; 0x00000001;); LG NetCast.Media-2011
        # Mozilla/5.0 (DirectFB; U; Linux 7630; en) AppleWebKit/531.2  (KHTML, like Gecko) Safari/531.2  LG Browser/4.1.4( 3D BDP NO_NUM; LGE; Media/ST600; LG NetCast.Media-2011
        # (LGSmartTV/1.0) AppleWebKit/534.23 OBIGO-T10/2.0
        #
        match = optimizedSearch(r'LG NetCast\.(?:TV|Media)-([0-9]*)', ua)
        if match:
            self.os['name'] = ''
            self.device['manufacturer'] = STRINGS_LG
            self.device['model'] = 'NetCast TV ' + match.group(1)
            self.device['type'] = 'television'
            self.device['identified'] = True

        match = optimizedSearch(r'LGSmartTV', ua)
        if match:
            self.os['name'] = ''
            self.device['manufacturer'] = STRINGS_LG
            self.device['model'] = 'Smart TV'
            self.device['type'] = 'television'
            self.device['identified'] = True



        # Toshiba Smart TV
        #
        # Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL953; 56.7.66.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH) ; en) AppleWebKit/534.1 (KHTML, like Gecko)
        # Mozilla/5.0 (DTV; TSBNetTV/T32013713.0203.7DD; TVwithVideoPlayer; like Gecko) NetFront/4.1 DTVNetBrowser/2.2 (000039;T32013713;0203;7DD) InettvBrowser/2.2 (000039;T32013713;0203;7DD)
        # Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; 40PX200; 0.7.3.0.; t12; ) ; Toshiba_TP/1.3.0 (+VIDEO_MP4+AUDIO_MPEG+AUDIO_MP4+VIDEO_X_MS_ASF+OFFLINEAPP) ; en) AppleWebKit/534.1 (KHTML, like Gec
        #

        if optimizedSearch(r'Toshiba_?TP\/', ua) or optimizedSearch(r'TSBNetTV\/', ua):
            self.os['name'] = ''
            self.device['manufacturer'] = 'Toshiba'
            self.device['model'] = 'Smart TV'
            self.device['type'] = 'television'
            self.device['identified'] = True



        # MachBlue XT
        #
        match = optimizedSearch(r'mbxtWebKit\/([0-9.]*)', ua)
        if match:
            self.os['name'] = ''
            self.browser['name'] = 'MachBlue XT'
            self.browser['version'] = Version({
                'value': match.group(1),
                'details': 2
            })
            self.device['type'] = 'television'



        # ADB
        #
        match = optimizedSearch(r'\(ADB; ([^\)]+)\)', ua)
        if match:
            self.os['name'] = ''
            self.device['manufacturer'] = 'ADB'
            self.device['model'] = (match.group(1).replace('ADB', '') + ' ' if match.group(1) is not 'Unknown' else '') + 'IPTV receiver'
            self.device['type'] = 'television'
            self.device['identified'] = True



        # MStar
        #
        if optimizedSearch(r'Mstar;OWB', ua):
            self.os['name'] = ''
            self.device['manufacturer'] = 'MStar'
            self.device['model'] = 'PVR'
            self.device['type'] = 'television'
            self.device['identified'] = True

            self.browser['name'] = 'Origyn Web Browser'



        # TechniSat
        #
        match = optimizedSearch(r'TechniSat ([^;]+);', ua)
        if match:
            self.os['name'] = ''
            self.device['manufacturer'] = 'TechniSat'
            self.device['model'] = match.group(1)
            self.device['type'] = 'television'
            self.device['identified'] = True



        # Technicolor
        #
        match = optimizedSearch(r'Technicolor_([^;]+);', ua)
        if match:
            self.os['name'] = ''
            self.device['manufacturer'] = 'Technicolor'
            self.device['model'] = match.group(1)
            self.device['type'] = 'television'
            self.device['identified'] = True



        # Winbox Evo2
        #
        match = optimizedSearch(r'Winbox Evo2', ua)
        if match:
            self.os['name'] = ''
            self.device['manufacturer'] = 'Winbox'
            self.device['model'] = 'Evo2'
            self.device['type'] = 'television'
            self.device['identified'] = True



        # Roku
        #
        match = optimizedSearch(r'^Roku\/DVP-([0-9]+)', ua)
        if match:
            self.device['manufacturer'] = 'Roku'
            self.device['type'] = 'television'
            tmpMap = {
                '2000': 'HD',
                '2050': 'XD',
                '2100': 'XDS',
                '2400': 'LT',
                '3000': '2 HD',
                '3050': '2 XD',
                '3100': '2 XS'
            }
            if match.group(1) in tmpMap:
                self.device['model'] = tmpMap[match.group(1)]

            self.device['identified'] = True

        match = optimizedSearch(r'HbbTV\/1.1.1 \([^;]*;\s*([^;]*)\s*;\s*([^;]*)\s*;', ua)
        if match:
            vendorName = match.group(1).trim()
            modelName = match.group(2).trim()

            if not self.device['manufacturer'] and vendorName is not '' and vendorName is not 'vendorName':
                tmpMap = {
                    'LGE': 'LG',
                    'TOSHIBA': 'Toshiba',
                    'smart': 'Smart',
                    'tv2n': 'TV2N',
                    '3000': '2 HD',
                    '3050': '2 XD',
                    '3100': '2 XS'
                }
                if vendorName in tmpMap:
                    self.device['model'] = tmpMap[vendorName]
                else:
                    self.device['manufacturer'] = vendorName
                # origin
                # switch (vendorName) {
                #     case 'LGE':
                #         self.device['manufacturer'] = 'LG';
                #         break;
                #     case 'TOSHIBA':
                #         self.device['manufacturer'] = 'Toshiba';
                #         break;
                #     case 'smart':
                #         self.device['manufacturer'] = 'Smart';
                #         break;
                #     case 'tv2n':
                #         self.device['manufacturer'] = 'TV2N';
                #         break;
                #     default:
                #         self.device['manufacturer'] = vendorName;
                # }

                if not self.device['model'] and modelName is not '' and modelName is not 'modelName':
                    tmpMap = {
                        'GLOBAL_PLAT3': 'NetCast TV',
                        'SmartTV2012': 'Smart TV 2012',
                        'videoweb': 'Videoweb'
                    }
                    if vendorName in tmpMap:
                        self.device['model'] = tmpMap[vendorName]
                    else:
                        self.device['manufacturer'] = vendorName
                    # origin
                    # switch (modelName) {
                    #     case 'GLOBAL_PLAT3':
                    #         self.device['model'] = 'NetCast TV';
                    #         break;
                    #     case 'SmartTV2012':
                    #         self.device['model'] = 'Smart TV 2012';
                    #         break;
                    #     case 'videoweb':
                    #         self.device['model'] = 'Videoweb';
                    #         break;
                    #     default:
                    #         self.device['model'] = modelName;
                    # }

                    if vendorName is 'Humax':
                        self.device['model'] = self.device['model'].upper()


                    self.device['identified'] = True
                    self.os['name'] = ''



            self.device['type'] = 'television'



        # Detect type based on common identifiers
        #

        if 'InettvBrowser' in ua:
            self.device['type'] = 'television'


        if 'MIDP' in ua:
            self.device['type'] = 'mobile'



        # Try to detect any devices based on common
        # locations of model ids
        #

        if not self.device['model'] and not self.device['manufacturer']:
            candidates = []

            if not optimizedSearch(r'^(Mozilla|Opera)', ua):
                match = optimizedSearch(r'^(?:MQQBrowser\/[0-9\.]+\/)?([^\s]+)', ua)
                if match:
                    tmp = match.group(1)
                    tmp = optimizedSub(r'_TD$', '', tmp, 1)
                    tmp = optimizedSub(r'_CMCC$', '', tmp, 1)
                    tmp = optimizedSub(r'[_ ]Mozilla$', '', tmp, 1)
                    tmp = optimizedSub(r' Linux$', '', tmp, 1)
                    tmp = optimizedSub(r' Opera$', '', tmp, 1)
                    tmp = optimizedSub(r'\/[0-9].*$', '', tmp, 1)

                    # match.group(1) = match.group(1).replace(/_TD$/, '')
                    # match.group(1) = match.group(1).replace(//, '')
                    # match.group(1) = match.group(1).replace(//, '')
                    # match.group(1) = match.group(1).replace(/ Linux$/, '')
                    # match.group(1) = match.group(1).replace(/ Opera$/, '')
                    # match.group(1) = match.group(1).replace(/\/[0-9].*$/, '')

                    candidates.append(tmp)

            match = optimizedSearch(r'[0-9]+x[0-9]+; ([^;]+)', ua)
            if match:
                candidates.append(match.group(1))

            match = optimizedSearch(r'[0-9]+X[0-9]+ ([^;\/\(\)]+)', ua)
            if match:
                candidates.append(match.group(1))

            match = optimizedSearch(r'Windows NT 5.1; ([^;]+); Windows Phone', ua)
            if match:
                candidates.append(match.group(1))

            match = optimizedSearch(r'\) PPC; (?:[0-9]+x[0-9]+; )?([^;\/\(\)]+)', ua)
            if match:
                candidates.append(match.group(1))

            match = optimizedSearch(r'\(([^;]+); U; Windows Mobile', ua)
            if match:
                candidates.append(match.group(1))

            match = optimizedSearch(r'Vodafone\/1.0\/([^\/]+)', ua)
            if match:
                candidates.append(match.group(1))

            match = optimizedSearch(r'\ ([^\s]+)$', ua)
            if match:
                candidates.append(match.group(1))

            for item in candidates:
            # for (i = 0; i < candidates.length; i++) {

                if not self.device['model'] and not self.device['manufacturer']:
                    model = cleanupModel(item)
                    result = False

                    if self.os['name'] is 'Android':
                        if model in ANDROID_MODELS:
                            self.device['manufacturer'] = ANDROID_MODELS[model][0]
                            self.device['model'] = ANDROID_MODELS[model][1]
                            if len(ANDROID_MODELS[model]) > 2:
                                self.device['type'] = ANDROID_MODELS[model][2]
                            self.device['identified'] = True

                            result = True



                    if not self.os['name'] or self.os['name'] is 'Windows' or self.os['name'] is 'Windows Mobile' or self.os['name'] is 'Windows CE':
                        if model in WINDOWS_MOBILE_MODELS:
                            self.device['manufacturer'] = WINDOWS_MOBILE_MODELS[model][0]
                            self.device['model'] = WINDOWS_MOBILE_MODELS[model][1]
                            self.device['type'] = 'mobile'
                            self.device['identified'] = True

                            if self.os['name'] is not 'Windows Mobile':
                                self.os['name'] = 'Windows Mobile'
                                self.os['version'] = None


                            result = True




                if not result:
                    tmpArr = [
                        {
                            'reg': '^GIONEE-([^\s]+)',
                            'manufacturer': 'Gionee'
                        },
                        {
                            'reg': '^HTC_?([^\/_]+)(?:\/|_|$)',
                            'manufacturer': STRINGS_HTC
                        },
                        {
                            'reg': '^HUAWEI-([^\/]*)',
                            'manufacturer': STRINGS_HUAWEI
                        },
                        {
                            'reg': '(?:^|\()LGE?(?:\/|-|_|\s)([^\s]*)',
                            'manufacturer': STRINGS_LG
                        },
                        {
                            'reg': '^MOT-([^\/_]+)(?:\/|_|$)',
                            'manufacturer': STRINGS_MOTOROLA
                        },
                        {
                            'reg': '^Motorola_([^\/_]+)(?:\/|_|$)',
                            'manufacturer': STRINGS_MOTOROLA
                        },
                        {
                            'reg': '^Nokia([^\/]+)(?:\/|$)',
                            'manufacturer': 'Nokia'
                        },
                        {
                            'reg': '^SonyEricsson([^\/_]+)(?:\/|_|$)',
                            'manufacturer': STRINGS_SONY_ERICSSON
                        },

                        {
                            'reg': '^SonyEricsson([^\/_]+)(?:\/|_|$)',
                            'manufacturer': STRINGS_SONY_ERICSSON
                        },
                        {
                            'reg': '^SonyEricsson([^\/_]+)(?:\/|_|$)',
                            'manufacturer': STRINGS_SONY_ERICSSON
                        }
                    ]
                    for row in tmpArr:
                        match = optimizedSearch(row['reg'], item)
                        if match:
                            self.device['manufacturer'] = row['manufacturer']
                            self.device['model'] = cleanupModel(match.group(1))
                            self.device['type'] = 'mobile'
                            self.device['identified'] = True

                            if row['manufacturer'] == 'Nokia' and not self.os['name']:
                                self.os['name'] = 'Series40'

                            break
                    # match = optimizedSearch(r'^HTC_?([^\/_]+)(?:\/|_|$)', item)
                    # if match:
                    #     self.device['manufacturer'] = STRINGS_HTC
                    #     self.device['model'] = cleanupModel(match.group(1))
                    #     self.device['type'] = 'mobile'
                    #     self.device['identified'] = True


                    # if match = //.exec(item):
                    #     self.device['manufacturer'] = STRINGS_HUAWEI
                    #     self.device['model'] = cleanupModel(match.group(1))
                    #     self.device['type'] = 'mobile'
                    #     self.device['identified'] = True


                    # if match = //.exec(item):
                    #     self.device['manufacturer'] = STRINGS_LG
                    #     self.device['model'] = cleanupModel(match.group(1))
                    #     self.device['type'] = 'mobile'
                    #     self.device['identified'] = True


                    # if match = //.exec(item):
                    #     self.device['manufacturer'] = STRINGS_MOTOROLA
                    #     self.device['model'] = cleanupModel(match.group(1))
                    #     self.device['type'] = 'mobile'
                    #     self.device['identified'] = True


                    # if match = //.exec(item):
                    #     self.device['manufacturer'] = STRINGS_MOTOROLA
                    #     self.device['model'] = cleanupModel(match.group(1))
                    #     self.device['type'] = 'mobile'
                    #     self.device['identified'] = True


                    # if match = //.exec(item):
                    #     self.device['manufacturer'] = 'Nokia'
                    #     self.device['model'] = cleanupModel(match.group(1))
                    #     self.device['type'] = 'mobile'
                    #     self.device['identified'] = True

                        # if not self.os['name']:
                        #     self.os['name'] = 'Series40'



                    # if match = //.exec(item):
                    #     self.device['manufacturer'] = STRINGS_SONY_ERICSSON
                    #     self.device['model'] = cleanupModel(match.group(1))
                    #     self.device['type'] = 'mobile'
                    #     self.device['identified'] = True

                    match = optimizedSearch(r'^SAMSUNG-([^\/_]+)(?:\/|_|$)', item)
                    if match:
                        self.device['manufacturer'] = STRINGS_SAMSUNG
                        self.device['model'] = cleanupModel(match.group(1))
                        self.device['type'] = 'mobile'

                        if self.os['name'] is 'Bada':
                            manufacturer = 'SAMSUNG'
                            model = cleanupModel(self.device['model'])

                            if manufacturer in BADA_MODELS and model in BADA_MODELS[manufacturer]:
                                self.device['manufacturer'] = BADA_MODELS[manufacturer][model][0]
                                self.device['model'] = BADA_MODELS[manufacturer][model][1]
                                self.device['identified'] = True

                        elif optimizedSearch(r'Jasmine\/([0-9.]*)', ua):
                            match = optimizedSearch(r'Jasmine\/([0-9.]*)', ua)
                            version = match.group(1)
                            manufacturer = 'SAMSUNG'
                            model = cleanupModel(self.device['model'])

                            if manufacturer in TOUCHWIZ_MODELS and model in TOUCHWIZ_MODELS[manufacturer]:
                                self.device['manufacturer'] = TOUCHWIZ_MODELS[manufacturer][model][0]
                                self.device['model'] = TOUCHWIZ_MODELS[manufacturer][model][1]
                                self.device['identified'] = True

                                self.os['name'] = 'Touchwiz'
                                self.os['version'] = Version({
                                    'value': '2.0'
                                })

                        elif optimizedSearch(r'Dolfin\/([0-9.]*)', ua):
                            match = optimizedSearch(r'Dolfin\/([0-9.]*)', ua)
                            version = match.group(1)
                            manufacturer = 'SAMSUNG'
                            model = cleanupModel(self.device['model'])

                            if manufacturer in BADA_MODELS and model in BADA_MODELS[manufacturer]:
                                self.device['manufacturer'] = BADA_MODELS[manufacturer][model][0]
                                self.device['model'] = BADA_MODELS[manufacturer][model][1]
                                self.device['identified'] = True

                                self.os['name'] = 'Bada'
                                tmpMap = {
                                    '2.0': '1.0',
                                    '2.2': '1.2',
                                    '3.0': '2.0',
                                }
                                if version in tmpMap:
                                    self.os['version'] = Version({
                                        'value': tmpMap[version]
                                    })
                                # origin


                            if manufacturer in TOUCHWIZ_MODELS and model in TOUCHWIZ_MODELS[manufacturer]:
                                self.device['manufacturer'] = TOUCHWIZ_MODELS[manufacturer][model][0]
                                self.device['model'] = TOUCHWIZ_MODELS[manufacturer][model][1]
                                self.device['identified'] = True

                                self.os['name'] = 'Touchwiz'

                                tmpMap = {
                                    '1.0': '1.0',
                                    '1.5': '2.0',
                                    '2.0': '3.0',
                                }
                                if version in tmpMap:
                                    self.os['version'] = Version({
                                        'value': tmpMap[version]
                                    })





            # }


        match = optimizedSearch(r'\((?:LG[-|\/])(.*) (?:Browser\/)?AppleWebkit', ua)
        if match:
            self.device['manufacturer'] = STRINGS_LG
            self.device['model'] = match.group(1)
            self.device['type'] = 'mobile'
            self.device['identified'] = True

        match = optimizedSearch(r'^Mozilla\/5.0 \((?:Nokia|NOKIA)(?:\s?)([^\)]+)\)UC AppleWebkit\(like Gecko\) Safari\/530$', ua)
        if match:
            self.device['manufacturer'] = 'Nokia'
            self.device['model'] = match.group(1)
            self.device['type'] = 'mobile'
            self.device['identified'] = True

            self.os['name'] = 'Series60'





        # Safari
        #

        if 'Safari' in ua:
            if self.os['name'] is 'iOS':
                self.browser['stock'] = True
                self.browser['hidden'] = True
                self.browser['name'] = 'Safari'
                self.browser['version'] = None




            if self.os['name'] is 'Mac OS X' or self.os['name'] is 'Windows':
                self.browser['name'] = 'Safari'
                self.browser['stock'] = self.os['name'] is 'Mac OS X'
                match = optimizedSearch(r'Version\/([0-9\.]+)', ua)
                if match:
                    self.browser['version'] = Version({
                        'value': match.group(1)
                    })

                match = optimizedSearch(r'AppleWebKit\/[0-9\.]+\+', ua)
                if match:
                    self.browser['name'] = 'WebKit Nightly Build'
                    self.browser['version'] = None





        # Internet Explorer
        #

        if 'MSIE' in ua:
            self.browser['name'] = 'Internet Explorer'

            if 'IEMobile' in ua or 'Windows CE' in ua or 'Windows Phone' in ua or 'WP7' in ua:
                self.browser['name'] = 'Mobile Internet Explorer'

            match = optimizedSearch(r'MSIE ([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })




        # Opera
        #
        if optimizedSearch(r'Opera', ua, re.I):
            self.browser['stock'] = False
            self.browser['name'] = 'Opera'
            match = optimizedSearch(r'Opera[\/| ]([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })

            match = optimizedSearch(r'Version\/([0-9.]*)', ua)
            if match:
                if float(match.group(1)) >= 10:
                    self.browser['version'] = Version({
                        'value': match.group(1)
                    })
                else:
                    self.browser['version'] = None



            if self.browser['version'] and 'Edition Labs' in ua:
                self.browser['version'].type = 'alpha'
                self.browser['channel'] = 'Labs'


            if self.browser['version'] and 'Edition Next' in ua:
                self.browser['version'].type = 'alpha'
                self.browser['channel'] = 'Next'


            if 'Opera Tablet' in ua:
                self.browser['name'] = 'Opera Mobile'
                self.device['type'] = 'tablet'


            if 'Opera Mobi' in ua:
                self.browser['name'] = 'Opera Mobile'
                self.device['type'] = 'mobile'

            match = optimizedSearch(r'Opera Mini;', ua)
            if match:
                self.browser['name'] = 'Opera Mini'
                self.browser['version'] = None
                self.browser['mode'] = 'proxy'
                self.device['type'] = 'mobile'

            match = optimizedSearch(r'Opera Mini\/(?:att\/)?([0-9.]*)', ua)
            if match:
                self.browser['name'] = 'Opera Mini'
                self.browser['version'] = Version({
                    'value': match.group(1),
                    'details': -1
                })
                self.browser['mode'] = 'proxy'
                self.device['type'] = 'mobile'


            if self.browser['name'] is 'Opera' and self.device['type'] is 'mobile':
                self.browser['name'] = 'Opera Mobile'
                if 'BER' in ua:
                    self.browser['name'] = 'Opera Mini'
                    self.browser['version'] = None



            if 'InettvBrowser' in ua:
                self.device['type'] = 'television'


            if 'Opera TV' in ua or 'Opera-TV' in ua:
                self.browser['name'] = 'Opera'
                self.device['type'] = 'television'


            if 'Linux zbov' in ua:
                self.browser['name'] = 'Opera Mobile'
                self.browser['mode'] = 'desktop'

                self.device['type'] = 'mobile'

                self.os['name'] = None
                self.os['version'] = None


            if 'Linux zvav' in ua:
                self.browser['name'] = 'Opera Mini'
                self.browser['version'] = None
                self.browser['mode'] = 'desktop'

                self.device['type'] = 'mobile'

                self.os['name'] = None
                self.os['version'] = None




        # Firefox
        #

        if 'Firefox' in ua:
            self.browser['stock'] = False
            self.browser['name'] = 'Firefox'
            match = optimizedSearch(r'Firefox\/([0-9ab.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })


            if self.browser['version'].type is 'alpha':
                self.browser['channel'] = 'Aurora'


            if self.browser['version'].type is 'beta':
                self.browser['channel'] = 'Beta'


            if 'Fennec' in ua:
                self.device['type'] = 'mobile'


            if 'Mobile; rv' in ua:
                self.device['type'] = 'mobile'


            if 'Tablet; rv' in ua:
                self.device['type'] = 'tablet'


            if self.device['type'] is 'mobile' or self.device['type'] is 'tablet':
                self.browser['name'] = 'Firefox Mobile'



        if 'Namoroka' in ua:
            self.browser['stock'] = False
            self.browser['name'] = 'Firefox'
            match = optimizedSearch(r'Namoroka\/([0-9ab.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })


            self.browser['channel'] = 'Namoroka'


        if 'Shiretoko' in ua:
            self.browser['stock'] = False
            self.browser['name'] = 'Firefox'
            match = optimizedSearch(r'Shiretoko\/([0-9ab.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })


            self.browser['channel'] = 'Shiretoko'


        if 'Minefield' in ua:
            self.browser['stock'] = False
            self.browser['name'] = 'Firefox'
            match = optimizedSearch(r'Minefield\/([0-9ab.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })


            self.browser['channel'] = 'Minefield'


        if 'Firebird' in ua:
            self.browser['stock'] = False
            self.browser['name'] = 'Firebird'
            match = optimizedSearch(r'Firebird\/([0-9ab.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })




        # SeaMonkey
        #

        if 'SeaMonkey' in ua:
            self.browser['stock'] = False
            self.browser['name'] = 'SeaMonkey'
            match = optimizedSearch(r'SeaMonkey\/([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })




        # Netscape
        #

        if 'Netscape' in ua:
            self.browser['stock'] = False
            self.browser['name'] = 'Netscape'
            match = optimizedSearch(r'Netscape[0-9]?\/([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })




        # Konqueror
        #

        if optimizedSearch(r'[k|K]onqueror/', ua):
            self.browser['name'] = 'Konqueror'
            match = optimizedSearch(r'[k|K]onqueror\/([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })




        # Chrome
        #
        match = optimizedSearch(r'(?:Chrome|CrMo|CriOS)\/([0-9.]*)', ua)
        if match:
            self.browser['stock'] = False
            self.browser['name'] = 'Chrome'
            self.browser['version'] = Version({
                'value': match.group(1)
            })

            tmp = '.'.join(match.group(1).split('.', 2))
            if self.os['name'] is 'Android':
                if tmp == '16.0.912':
                    self.browser['channel'] = 'Beta'
                elif tmp == '18.0.1025':
                    self.browser['version'].details = 1
                else:
                    self.browser['channel'] = 'Nightly'
                # switch () {
                #     case '16.0.912':
                #         self.browser['channel'] = 'Beta';
                #         break;
                #     case '18.0.1025':
                #         self.browser['version'].details = 1;
                #         break;
                #     default:
                #         self.browser['channel'] = 'Nightly';
                #         break;
                # }
            else:
                # origin
                # switch (match.group(1).split('.', 3).join('.')) {
                tmpArr = [
                    '0.2.149',
                    '0.3.154',
                    '0.4.154',
                    '1.0.154',
                    '2.0.172',
                    '3.0.195',
                    '4.0.249',
                    '4.1.249',
                    '5.0.375',
                    '6.0.472',
                    '7.0.517',
                    '8.0.552',
                    '9.0.597',
                    '10.0.648',
                    '11.0.696',
                    '12.0.742',
                    '13.0.782',
                    '14.0.835',
                    '15.0.874',
                    '16.0.912',
                    '17.0.963',
                    '18.0.1025',
                    '19.0.1084',
                    '20.0.1132',
                    '21.0.1180'
                ]
                if tmp in tmpArr:
                    if self.browser['version'].minor == 0:
                        self.browser['version'].details = 1
                    else:
                        self.browser['version'].details = 2
                else:
                    self.browser['channel'] = 'Nightly'



        # Chrome Frame
        #

        if 'chromeframe' in ua:
            self.browser['stock'] = False
            self.browser['name'] = 'Chrome Frame'
            match = optimizedSearch(r'chromeframe\/([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })




        # Chromium
        #

        if 'Chromium' in ua:
            self.browser['stock'] = False
            self.browser['channel'] = ''
            self.browser['name'] = 'Chromium'
            match = optimizedSearch(r'Chromium\/([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })




        # BrowserNG
        #

        if 'BrowserNG' in ua:
            self.browser['name'] = 'Nokia Browser'
            match = optimizedSearch(r'BrowserNG\/([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1),
                    'details': 3,
                    builds: False
                })

        # CFNetwork ios网络库
        if 'CFNetwork' in ua:
            self.browser['name'] = 'CFNetwork'
            match = optimizedSearch(r'CFNetwork\/([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1),
                    'details': 3
                })

        # Nokia Browser
        #

        if 'NokiaBrowser' in ua:
            self.browser['name'] = 'Nokia Browser'
            match = optimizedSearch(r'NokiaBrowser\/([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1),
                    'details': 3
                })




        # MicroB
        #

        if optimizedSearch(r'Maemo[ |_]Browser', ua):
            self.browser['name'] = 'MicroB'
            match = optimizedSearch(r'Maemo[ |_]Browser[ |_]([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1),
                    'details': 3
                })





        # NetFront
        #

        if 'NetFront' in ua:
            self.browser['name'] = 'NetFront'
            self.device['type'] = 'mobile'
            match = optimizedSearch(r'NetFront\/([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })


            if 'InettvBrowser' in ua:
                self.device['type'] = 'television'




        # Silk
        #

        if 'Silk' in ua:
            if 'Silk-Accelerated' in ua:
                self.browser['name'] = 'Silk'
                match = optimizedSearch(r'Silk\/([0-9.]*)', ua)
                if match:
                    self.browser['version'] = Version({
                        'value': match.group(1),
                        'details': 2
                    })


                self.device['manufacturer'] = 'Amazon'
                self.device['model'] = 'Kindle Fire'
                self.device['type'] = 'tablet'
                self.device['identified'] = True

                if self.os['name'] is not 'Android':
                    self.os['name'] = 'Android'
                    self.os['version'] = None





        # Dolfin
        #

        if 'Dolfin' in ua:
            self.browser['name'] = 'Dolfin'
            match = optimizedSearch(r'Dolfin\/([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })





        # Iris
        #

        if 'Iris' in ua:
            self.browser['name'] = 'Iris'

            self.device['type'] = 'mobile'
            self.device['model'] = None
            self.device['manufacturer'] = None

            self.os['name'] = 'Windows Mobile'
            self.os['version'] = None
            match = optimizedSearch(r'Iris\/([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })

            match = optimizedSearch(r' WM([0-9]) ', ua)
            if match:
                self.os['version'] = Version({
                    'value': match.group(1) + '.0'
                })
            else:
                self.browser['mode'] = 'desktop'




        # Jasmine
        #

        if 'Jasmine' in ua:
            self.browser['name'] = 'Jasmine'
            match = optimizedSearch(r'Jasmine\/([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })




        # Boxee
        #

        if 'Boxee' in ua:
            self.browser['name'] = 'Boxee'
            self.device['type'] = 'television'
            match = optimizedSearch(r'Boxee\/([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })




        # Espial
        #

        if 'Espial' in ua:
            self.browser['name'] = 'Espial'

            self.os['name'] = ''
            self.os['version'] = None

            if self.device['type'] is not 'television':
                self.device['type'] = 'television'
                self.device['model'] = None
                self.device['manufacturer'] = None

            match = optimizedSearch(r'Espial\/([0-9.]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })




        # ANT Galio
        #
        match = optimizedSearch(r'ANTGalio\/([0-9.]*)', ua)
        if match:
            self.browser['name'] = 'ANT Galio'
            self.browser['version'] = Version({
                'value': match.group(1),
                'details': 3
            })
            self.device['type'] = 'television'



        # NetFront NX
        #
        match = optimizedSearch(r'NX\/([0-9.]*)', ua)
        if match:
            self.browser['name'] = 'NetFront NX'
            self.browser['version'] = Version({
                'value': match.group(1),
                'details': 2
            })
            match = optimizedSearch(r'DTV', ua, re.I)
            if match:
                self.device['type'] = 'television'
            elif optimizedSearch(r'mobile', ua, re.I):
                self.device['type'] = 'mobile'
            else:
                self.device['type'] = 'desktop'


            self.os['name'] = None
            self.os['version'] = None



        # Obigo
        #
        match = optimizedSearch(r'Obigo', ua, re.I)
        if match:
            self.browser['name'] = 'Obigo'
            match = optimizedSearch(r'Obigo\/([0-9.]*)', ua, re.I)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1)
                })

            match = optimizedSearch(r'Obigo\/([A-Z])([0-9.]*)', ua, re.I)
            if match:
                self.browser['name'] = 'Obigo ' + match.group(1)
                self.browser['version'] = Version({
                    'value': match.group(2)
                })

            match = optimizedSearch(r'Obigo-([A-Z])([0-9.]*)\/', ua, re.I)
            if match:
                self.browser['name'] = 'Obigo ' + match.group(1)
                self.browser['version'] = Version({
                    'value': match.group(2)
                })




        # UC Web
        #

        if 'UCWEB' in ua:
            self.browser['stock'] = False
            self.browser['name'] = 'UC Browser'
            match = optimizedSearch(r'UCWEB([0-9]*[.][0-9]*)', ua)
            if match:
                self.browser['version'] = Version({
                    'value': match.group(1),
                    'details': 3
                })


            if self.os['name'] is 'Linux':
                self.os['name'] = ''


            self.device['type'] = 'mobile'
            match = optimizedSearch(r'^IUC \(U;\s?iOS ([0-9\.]+);', ua)
            if match:
                self.os['name'] = 'iOS'
                self.os['version'] = Version({
                    'value': match.group(1)
                })

            match = optimizedSearch(r'^JUC \(Linux; U; ([0-9\.]+)[^;]*; [^;]+; ([^;]*[^\s])\s*; [0-9]+\*[0-9]+\)', ua)
            if match:
                model = cleanupModel(match.group(2))

                self.os['name'] = 'Android'
                self.os['version'] = Version({
                    'value': match.group(1)
                })

                if model in ANDROID_MODELS:
                    self.device['manufacturer'] = ANDROID_MODELS[model][0]
                    self.device['model'] = ANDROID_MODELS[model][1]
                    if len(ANDROID_MODELS[model]) > 2:
                        self.device['type'] = ANDROID_MODELS[model][2]
                    self.device['identified'] = True



        match = optimizedSearch(r'\) UC ', ua)
        if match:
            self.browser['stock'] = False
            self.browser['name'] = 'UC Browser'

        match = optimizedSearch(r'UCBrowser\/([0-9.]*)', ua)
        if match:
            self.browser['stock'] = False
            self.browser['name'] = 'UC Browser'
            self.browser['version'] = Version({
                'value': match.group(1),
                'details': 2
            })



        # NineSky
        #
        match = optimizedSearch(r'Ninesky(?:-android-mobile(?:-cn)?)?\/([0-9.]*)', ua)
        if match:
            self.browser['name'] = 'NineSky'
            self.browser['version'] = Version({
                'value': match.group(1)
            })

            if self.os['name'] is not 'Android':
                self.os['name'] = 'Android'
                self.os['version'] = None

                self.device['manufacturer'] = None
                self.device['model'] = None




        # Skyfire
        #
        match = optimizedSearch(r'Skyfire\/([0-9.]*)', ua)
        if match:
            self.browser['name'] = 'Skyfire'
            self.browser['version'] = Version({
                'value': match.group(1)
            })

            self.device['type'] = 'mobile'

            self.os['name'] = 'Android'
            self.os['version'] = None



        # Dolphin HD
        #
        match = optimizedSearch(r'DolphinHDCN\/([0-9.]*)', ua)
        if match:
            self.browser['name'] = 'Dolphin'
            self.browser['version'] = Version({
                'value': match.group(1)
            })

            self.device['type'] = 'mobile'

            if self.os['name'] is not 'Android':
                self.os['name'] = 'Android'
                self.os['version'] = None


        match = optimizedSearch(r'Dolphin\/INT', ua)
        if match:
            self.browser['name'] = 'Dolphin'
            self.device['type'] = 'mobile'



        # QQ Browser
        #
        match = optimizedSearch(r'(M?QQBrowser)\/([0-9.]*)', ua)
        if match:
            self.browser['name'] = 'QQ Browser'

            version = match.group(2)
            if optimizedSearch(r'^[0-9][0-9]$', version):
                version = version[0] + '.' + version[1]

            self.browser['version'] = Version({
                'value': version,
                'details': 2
            })
            self.browser['channel'] = ''

            if not self.os['name'] and match.group(1) is 'QQBrowser':
                self.os['name'] = 'Windows'




        # iBrowser
        #
        match = optimizedSearch(r'(iBrowser)\/([0-9.]*)', ua)
        if match:
            self.browser['name'] = 'iBrowser'

            version = match.group(2)
            if optimizedSearch(r'[0-9][0-9]', version):
                version = version[0] + '.' + version[1]

            self.browser['version'] = Version({
                'value': version,
                'details': 2
            })
            self.browser['channel'] = ''



        # Puffin
        #
        match = optimizedSearch(r'Puffin\/([0-9.]*)', ua)
        if match:
            self.browser['name'] = 'Puffin'
            self.browser['version'] = Version({
                'value': match.group(1),
                'details': 2
            })

            self.device['type'] = 'mobile'

            if self.os['name'] is 'Linux':
                self.os['name'] = None
                self.os['version'] = None




        # 360 Extreme Explorer
        #

        if '360EE' in ua:
            self.browser['stock'] = False
            self.browser['name'] = '360 Extreme Explorer'
            self.browser['version'] = None



        # Midori
        #
        match = optimizedSearch(r'Midori\/([0-9.]*)', ua)
        if match:
            self.browser['name'] = 'Midori'
            self.browser['version'] = Version({
                'value': match.group(1)
            })

            if self.os['name'] is not 'Linux':
                self.os['name'] = 'Linux'
                self.os['version'] = None


            self.device['manufacturer'] = None
            self.device['model'] = None
            self.device['type'] = 'desktop'


        # WebKit
        match = optimizedSearch(r'WebKit\/([0-9.]*)', ua, re.I)
        if match:
            self.engine['name'] = 'Webkit'
            self.engine['version'] = Version({
                'value': match.group(1)
            })

        match = optimizedSearch(r'Browser\/AppleWebKit([0-9.]*)', ua, re.I)
        if match:
            self.engine['name'] = 'Webkit'
            self.engine['version'] = Version({
                'value': match.group(1)
            })



        # KHTML
        #
        match = optimizedSearch(r'KHTML\/([0-9.]*)', ua)
        if match:
            self.engine['name'] = 'KHTML'
            self.engine['version'] = Version({
                'value': match.group(1)
            })



        # Gecko
        #

        if 'Gecko' in ua and not optimizedSearch(r'like Gecko', ua, re.I):
            self.engine['name'] = 'Gecko'

            match = optimizedSearch(r'; rv:([^\)]+)\)', ua)
            if match:
                self.engine['version'] = Version({
                    'value': match.group(1)
                })




        # Presto
        #
        match = optimizedSearch(r'Presto\/([0-9.]*)', ua)
        if match:
            self.engine['name'] = 'Presto'
            self.engine['version'] = Version({
                'value': match.group(1)
            })



        # Trident
        #
        match = optimizedSearch(r'Trident\/([0-9.]*)', ua)
        if match:
            self.engine['name'] = 'Trident'
            self.engine['version'] = Version({
                'value': match.group(1)
            })

            if self.browser['name'] is 'Internet Explorer':
                if parseVersion(self.engine['version']) is 6 and float(str(self.browser['version'])) < 10:
                    self.browser['version'] = Version({
                        'value': '10.0'
                    })
                    self.browser['mode'] = 'compat'


                if parseVersion(self.engine['version']) is 5 and float(str(self.browser['version'])) < 9:
                    self.browser['version'] = Version({
                        'value': '9.0'
                    })
                    self.browser['mode'] = 'compat'


                if parseVersion(self.engine['version']) is 4 and float(str(self.browser['version'])) < 8:
                    self.browser['version'] = Version({
                        'value': '8.0'
                    })
                    self.browser['mode'] = 'compat'



            if self.os['name'] is 'Windows Phone':
                if parseVersion(self.engine['version']) is 5 and float(self.os['version']) < 7.5:
                    self.os['version'] = Version({
                        'value': '7.5'
                    })






        # Corrections
        #

        if self.os['name'] is 'Android' and self.browser['stock']:
            self.browser['hidden'] = True


        if self.os['name'] is 'iOS' and self.browser['name'] is 'Opera Mini':
            self.os['version'] = None


        if self.browser['name'] is 'Midori' and self.engine['name'] is not 'Webkit':
            self.engine['name'] = 'Webkit'
            self.engine['version'] = None


        if self.device['type'] is 'television' and self.browser['name'] is 'Opera':
            self.browser['name'] = 'Opera Devices'
            tmpMap = {
                '2.10': 3.2,
                '2.9': 3.1,
                '2.8': 3.0,
                '2.7': 2.9,
                '2.6': 2.8,
                '2.4': 10.3,
                '2.3': 10,
                '2.2': 9.7,
                '2.1': 9.6
            }
            if self.engine['version'].original in tmpMap:
                self.browser['version'] = Version({
                    'value': tmpMap[self.engine['version'].original]
                })
            else:
                self.browser['version'] = None

            self.os['name'] = None
            self.os['version'] = None


        # Camouflage
        #

        if self.options['detectCamouflage']:
            match = optimizedSearch(r'Mac OS X 10_6_3; ([^;]+); [a-z]{2}-(?:[a-z]{2})?\)', ua)
            if match:
                self.browser['name'] = ''
                self.browser['version'] = None
                self.browser['mode'] = 'desktop'

                self.os['name'] = 'Android'
                self.os['version'] = None

                self.engine['name'] = 'Webkit'
                self.engine['version'] = None

                self.device['model'] = match.group(1)
                self.device['type'] = 'mobile'

                model = cleanupModel(self.device['model'])
                if model in ANDROID_MODELS:
                    self.device['manufacturer'] = ANDROID_MODELS[model][0]
                    self.device['model'] = ANDROID_MODELS[model][1]
                    if len(ANDROID_MODELS[model]) > 2:
                        self.device['type'] = ANDROID_MODELS[model][2]
                    self.device['identified'] = True


                self.features.append('foundDevice')

            match = optimizedSearch(r'Linux Ventana; [a-z]{2}-[a-z]{2}; (.+) Build', ua)
            if match:
                self.browser['name'] = ''
                self.browser['version'] = None
                self.browser['mode'] = 'desktop'

                self.os['name'] = 'Android'
                self.os['version'] = None

                self.engine['name'] = 'Webkit'
                self.engine['version'] = None

                self.device['model'] = match.group(1)
                self.device['type'] = 'mobile'

                model = cleanupModel(self.device['model'])
                if model in ANDROID_MODELS:
                    self.device['manufacturer'] = ANDROID_MODELS[model][0]
                    self.device['model'] = ANDROID_MODELS[model][1]
                    if len(ANDROID_MODELS[model]) > 2:
                        self.device['type'] = ANDROID_MODELS[model][2]
                    self.device['identified'] = True


                self.features.append('foundDevice')


            if self.browser['name'] is 'Safari':
                # match = optimizedSearch(r'AppleWebKit\/([0-9]+.[0-9]+)', ua, re.I).group(1)
                # optimizedSearch(r'Safari\/([0-9]+.[0-9]+)', ua, re.I).group(1)
                if self.os['name'] is not 'iOS' and optimizedSearch(r'AppleWebKit\/([0-9]+.[0-9]+)', ua, re.I).group(1) is not optimizedSearch(r'Safari\/([0-9]+.[0-9]+)', ua, re.I).group(1):
                    self.features.append('safariMismatch')
                    self.camouflage = True

                # match = optimizedSearch(r'^Mozilla', ua)
                if self.os['name'] is 'iOS' and not optimizedSearch(r'^Mozilla', ua):
                    self.features.append('noMozillaPrefix')
                    self.camouflage = True

                # match = optimizedSearch(r'Version\/[0-9\.]+', ua)
                if not optimizedSearch(r'Version\/[0-9\.]+', ua):
                    self.features.append('noVersion')
                    self.camouflage = True



            if self.browser['name'] is 'Chrome':
                match = optimizedSearch(r'(?:Chrome|CrMo|CriOS)\/([0-9]{1,2}\.[0-9]\.[0-9]{3,4}\.[0-9]+)', ua)
                if not match:
                    self.features.append('wrongVersion')
                    self.camouflage = True



            # 得运行在浏览器里才行的
            # if self.options.useFeatures:
                """If it claims not to be Trident, but it is probably Trident running camouflage mode"""
                # if window.ActiveXObject:
                #     self.features.append('trident')

                #     if self.engine['name'] and self.engine['name'] is not 'Trident':
                #         self.camouflage = self.browser['name'] is None or self.browser['name'] is not 'Maxthon'



                """If it claims not to be Opera, but it is probably Opera running camouflage mode"""
                # if window.opera:
                #     self.features.append('presto')

                #     if self.engine['name'] and self.engine['name'] is not 'Presto':
                #         self.camouflage = True


                #     if self.browser['name'] is 'Internet Explorer':
                #         self.camouflage = True



                """If it claims not to be Gecko, but it is probably Gecko running camouflage mode"""
                # if 'getBoxObjectFor' in document or 'mozInnerScreenX' in window:
                #     self.features.append('gecko')

                #     if self.engine['name'] and self.engine['name'] is not 'Gecko':
                #         self.camouflage = True


                #     if self.browser['name'] is 'Internet Explorer':
                #         self.camouflage = True



                """If it claims not to be Webkit, but it is probably Webkit running camouflage mode"""
                # if 'WebKitCSSMatrix' in window or 'WebKitPoint' in window or 'webkitStorageInfo' in window or 'webkitURL' in window:
                #     self.features.append('webkit')

                #     if self.engine['name'] and self.engine['name'] is not 'Webkit':
                #         self.camouflage = True


                #     if self.browser['name'] is 'Internet Explorer':
                #         self.camouflage = True





                """If it claims to be Safari and uses V8, it is probably an Android device running camouflage mode"""
                # if self.engine['name'] is 'Webkit' and ({}.toString).toString().indexOf('\n') is -1:
                #     self.features.append('v8')

                #     if self.browser is not None and self.browser['name'] is 'Safari':
                #         self.camouflage = True





                """If we have an iPad that is not 768 x 1024, we have an imposter"""
                # if self.device['model'] === 'iPad':
                #     if (screen.width is not 0 and screen.height is not 0) and (screen.width is not 768 and screen.height is not 1024) and (screen.width is not 1024 and screen.height is not 768):
                #         self.features.append('sizeMismatch')
                #         self.camouflage = True



                # """If we have an iPhone or iPod that is not 320 x 480, we have an imposter"""
                # if self.device['model'] === 'iPhone'or self.device['model'] === 'iPod':
                #     if (screen.width is not 0 and screen.height is not 0) and (screen.width is not 320 and screen.height is not 480) and (screen.width is not 480 and screen.height is not 320):
                #         self.features.append('sizeMismatch')
                #         self.camouflage = True




                # if self.os['name'] is 'iOS' and self.os['version']:

                    # if self.os['version'].isOlder('4.0') and 'sandbox' in document.createElement('iframe'):
                    #     self.features.append('foundSandbox')
                    #     self.camouflage = True


                    # if self.os['version'].isOlder('4.2') and 'WebSocket' in window:
                    #     self.features.append('foundSockets')
                    #     self.camouflage = True


                    # if self.os['version'].isOlder('5.0') and not not  window.Worker:
                    #     self.features.append('foundWorker')
                    #     self.camouflage = True


                    # if self.os['version'].isNewer('2.1') and not window.applicationCache:
                    #     self.features.append('noAppCache')
                    #     self.camouflage = True



                # if self.os['name'] is not 'iOS' and self.browser['name'] is 'Safari' and self.browser['version']:

                #     if self.browser['version'].isOlder('4.0') and not not  window.applicationCache:
                #         self.features.append('foundAppCache')
                #         self.camouflage = True


                #     if self.browser['version'].isOlder('4.1') and not not  (window.history and history.pushState):
                #         self.features.append('foundHistory')
                #         self.camouflage = True


                #     if self.browser['version'].isOlder('5.1') and not not  document.documentElement.webkitRequestFullScreen:
                #         self.features.append('foundFullscreen')
                #         self.camouflage = True


                #     if self.browser['version'].isOlder('5.2') and 'FileReader' in window:
                #         self.features.append('foundFileReader')
                #         self.camouflage = True





# };

def cleanupModel(s=''):
    # s = typeof s is 'undefined' ? '' : s

    s = optimizedSub(r'_TD$', '', s, 1)
    s = optimizedSub(r'_CMCC$', '', s, 1)

    s = optimizedSub(r'_', ' ', s)
    s = optimizedSub(r'^\s+|\s+$', '', s)
    s = optimizedSub(r'\/[^/]+$', '', s, 1)
    s = optimizedSub(r'\/[^/]+ Android\/.*', '', s, 1)

    s = optimizedSub(r'^tita on ', '', s, 1)
    s = optimizedSub(r'^Android on ', '', s, 1)
    s = optimizedSub(r'^Android for ', '', s, 1)
    s = optimizedSub(r'^ICS AOSP on ', '', s, 1)
    s = optimizedSub(r'^Full AOSP on ', '', s, 1)
    s = optimizedSub(r'^Full Android on ', '', s, 1)
    s = optimizedSub(r'^Full Cappuccino on ', '', s, 1)
    s = optimizedSub(r'^Full MIPS Android on ', '', s, 1)
    s = optimizedSub(r'^Full Android', '', s, 1)

    s = optimizedSub(r'^Acer ?', '', s, 1, re.I)
    s = optimizedSub(r'^Iconia ', '', s, 1)
    s = optimizedSub(r'^Ainol ', '', s, 1)
    s = optimizedSub(r'^Coolpad ?', 'Coolpad ', s, 1, re.I)
    s = optimizedSub(r'^ALCATEL ', '', s, 1)
    s = optimizedSub(r'^Alcatel OT-(.*)', 'one touch $1', s, 1)
    s = optimizedSub(r'^YL-', '', s, 1)
    s = optimizedSub(r'^Novo7 ?', 'Novo7 ', s, 1, re.I)
    s = optimizedSub(r'^GIONEE ', '', s, 1)
    s = optimizedSub(r'^HW-', '', s, 1)
    s = optimizedSub(r'^Huawei[ -]', 'Huawei ', s, 1, re.I)
    s = optimizedSub(r'^SAMSUNG[ -]', '', s, 1, re.I)
    s = optimizedSub(r'^SonyEricsson', '', s, 1)
    s = optimizedSub(r'^Lenovo Lenovo', 'Lenovo', s, 1)
    s = optimizedSub(r'^LNV-Lenovo', 'Lenovo', s, 1)
    s = optimizedSub(r'^Lenovo-', 'Lenovo ', s, 1)
    s = optimizedSub(r'^(LG)[ _\/]', '$1-', s, 1)
    s = optimizedSub(r'^(HTC.*)\s(?:v|V)?[0-9.]+$', '$1', s, 1)
    s = optimizedSub(r'^(HTC)[-\/]', '$1 ', s, 1)
    s = optimizedSub(r'^(HTC)([A-Z][0-9][0-9][0-9])', '$1 $2', s, 1)
    s = optimizedSub(r'^(Motorola[\s|-])', '', s, 1)
    s = optimizedSub(r'^(Moto|MOT-)', '', s, 1)

    s = optimizedSub(r'-?(orange(-ls)?|vodafone|bouygues)$', '', s, 1, re.I)
    s = optimizedSub(r'http:\/\/.+$', '', s, 1, re.I)

    s = optimizedSub(r'^\s+|\s+$', '', s)

    return s


def parseVersion(version):
    version = str(version) # .toString()
    components = version.split('.')
    major = components.pop(0)
    return float(major + '.' + ''.join(components))





# -*- coding: utf8 -*-
import sys
import re
import os
import json
from imp import reload
reload(sys)
if sys.version_info < (3, 0):
    sys.setdefaultencoding('utf-8')
sys.path.append('./')
from useragentBase import *

tmpData = {
    'ua': '',
    'match': None
}


def getMatch(string=None, reg=None, i=False):
    if not string:
        return tmpData['match']
    if i:
        tmpData['match'] = optimizedSearch(reg, string, re.I)
    else:
        tmpData['match'] = optimizedSearch(reg, string)
    return tmpData['match']


def parseUA(ua):
    tmpData['ua'] = ua
    uaData = UA(ua).getUaInfo()

    if not ua:
        return uaData

    # match = ''
    # tmpMatch = ''
    # handle mobile device
    if uaData['device']['type'] == 'mobile' or uaData['device']['type'] == 'tablet':
        # get manufacturer through the key words
        match = optimizedSearch(r'(ZTE|Samsung|Motorola|HTC|Coolpad|Huawei|Lenovo|LG|Sony Ericsson|Oppo|TCL|Vivo|Sony|Meizu|Nokia)', ua, re.I)
        if match:
            uaData['device']['manufacturer'] = match.group(1)
            if uaData['device']['model'] and match.group(1) in uaData['device']['model']:
                uaData['device']['model'] = uaData['device']['model'].replace(match.group(1), '')


        # handle Apple
        # 苹果就这3种iPod iPad iPhone
        if getMatch(ua, r'(iPod|iPad|iPhone)', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Apple'
            uaData['device']['model'] = match.group(1)

        # handle Samsung
        # 特殊型号可能以xxx-开头 或者直接空格接型号 兼容build结尾或直接)结尾
        # Galaxy nexus才是三星 nexus是google手机
        # 三星手机类型：galaxy xxx|SM-xxx|GT-xxx|SCH-xxx|SGH-xxx|SPH-xxx|SHW-xxx  若这些均未匹配到，则启用在中关村在线爬取到的机型白名单进行判断
        elif getMatch(ua, r'[-\s](Galaxy[\s\-_]nexus|Galaxy[\s\-_]\w*[\s\-_]\w*|Galaxy[\s\-_]\w*|SM-\w*|GT-\w*|s[cgp]h-\w*|shw-\w*|ATIV|i9070|omnia|s7568|A3000|A3009|A5000|A5009|A7000|A7009|A8000|C101|C1116|C1158|E400|E500F|E7000|E7009|G3139D|G3502|G3502i|G3508|G3508J|G3508i|G3509|G3509i|G3558|G3559|G3568V|G3586V|G3589W|G3606|G3608|G3609|G3812|G388F|G5108|G5108Q|G5109|G5306W|G5308W|G5309W|G550|G600|G7106|G7108|G7108V|G7109|G7200|G720NO|G7508Q|G7509|G8508S|G8509V|G9006V|G9006W|G9008V|G9008W|G9009D|G9009W|G9198|G9200|G9208|G9209|G9250|G9280|I535|I679|I739|I8190N|I8262|I879|I879E|I889|I9000|I9060|I9082|I9082C|I9082i|I9100|I9100G|I9108|I9128|I9128E|I9128i|I9152|I9152P|I9158|I9158P|I9158V|I9168|I9168i|I9190|I9192|I9195|I9195I|I9200|I9208|I9220|I9228|I9260|I9268|I9300|I9300i|I9305|I9308|I9308i|I939|I939D|I939i|I9500|I9502|I9505|I9507V|I9508|I9508V|I959|J100|J110|J5008|J7008|N7100|N7102|N7105|N7108|N7108D|N719|N750|N7505|N7506V|N7508V|N7509V|N900|N9002|N9005|N9006|N9008|N9008S|N9008V|N9009|N9100|N9106W|N9108V|N9109W|N9150|N916|N9200|P709|P709E|P729|S6358|S7278|S7278U|S7562C|S7562i|S7898i|b9388)[\s\)]', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Samsung'
            # 解决移动联通等不同发行版导致的机型不同问题
            # 特征：[A-Z]+[0-9]+[A-Z]*, 例如 G9006 G9006V 其实应该是G9006 另外三星只保留3位
            tmp = match.group(1)
            tmp = optimizedSub(r'Galaxy S VI', 'Galaxy S6', tmp, 1, re.I)
            tmp = optimizedSub(r'Galaxy S V', 'Galaxy S5', tmp, 1, re.I)
            tmp = optimizedSub(r'Galaxy S IV', 'Galaxy S4', tmp, 1, re.I)
            tmp = optimizedSub(r'Galaxy s III', 'Galaxy S3', tmp, 1, re.I)
            tmp = optimizedSub(r'Galaxy S II', 'Galaxy S2', tmp, 1, re.I)
            tmp = optimizedSub(r'Galaxy S I', 'Galaxy S1', tmp, 1, re.I)
            uaData['device']['model'] = optimizedSub(r'([a-z]+[0-9]{3})[0-9]?[a-z]*', r'\1', tmp, 1, re.I)
            # match.group(1).replace(/Galaxy S VI/i, 'Galaxy S6')
            #     .replace(/Galaxy S V/i, 'Galaxy S5')
            #     .replace(/Galaxy S IV/i, 'Galaxy S4')
            #     .replace(/Galaxy s III/i, 'Galaxy S3')
            #     .replace(/Galaxy S II/i, 'Galaxy S2')
            #     .replace(/Galaxy S I/i, 'Galaxy S1')
            #     .replace(//i, '$1')

        # 针对三星已经匹配出的数据做处理
        elif uaData['device']['manufacturer'] and uaData['device']['manufacturer'].lower() == 'samsung' and uaData['device']['model']:
            tmp = uaData['device']['model']
            tmp = optimizedSub(r'Galaxy S VI', 'Galaxy S6', tmp, 1, re.I)
            tmp = optimizedSub(r'Galaxy S V', 'Galaxy S5', tmp, 1, re.I)
            tmp = optimizedSub(r'Galaxy S IV', 'Galaxy S4', tmp, 1, re.I)
            tmp = optimizedSub(r'Galaxy s III', 'Galaxy S3', tmp, 1, re.I)
            tmp = optimizedSub(r'Galaxy S II', 'Galaxy S2', tmp, 1, re.I)
            tmp = optimizedSub(r'Galaxy S I', 'Galaxy S1', tmp, 1, re.I)
            uaData['device']['model'] = optimizedSub(r'([a-z]+[0-9]{3})[0-9]?[a-z]*', r'\1', tmp, 1, re.I)

            # uaData['device']['model'] = uaData['device']['model'].replace(/Galaxy S VI/i, 'Galaxy S6')
            #     .replace(/Galaxy S V/i, 'Galaxy S5')
            #     .replace(/Galaxy S IV/i, 'Galaxy S4')
            #     .replace(/Galaxy s III/i, 'Galaxy S3')
            #     .replace(/Galaxy S II/i, 'Galaxy S2')
            #     .replace(/Galaxy S I/i, 'Galaxy S1')
            #     .replace(/([a-z]+[0-9]{3})[0-9]?[a-z]*/i, '$1')

        # handle Huawei
        # 兼容build结尾或直接)结尾
        # 华为机型特征：Huawei[\s\-_](\w*[-_]?\w*)  或者以 7D-  ALE-  CHE-等开头
        elif getMatch(ua, r'(Huawei[\s\-_](\w*[-_]?\w*)|\s(7D-\w*|ALE-\w*|ATH-\w*|CHE-\w*|CHM-\w*|Che1-\w*|Che2-\w*|D2-\w*|G616-\w*|G620S-\w*|G621-\w*|G660-\w*|G750-\w*|GRA-\w*|Hol-\w*|MT2-\w*|MT7-\w*|PE-\w*|PLK-\w*|SC-\w*|SCL-\w*|H60-\w*|H30-\w*)[\s\)])', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Huawei'
            if match.group(2):
                uaData['device']['model'] = match.group(2)

            elif match.group(3):
                uaData['device']['model'] = match.group(3)

            # 解决移动联通等不同发行版导致的机型不同问题
            # 特征：xxx-[A-Z][0-9]+ 例如  H30-L01  H30-L00  H30-L20  都应该是 H30-L
            # h30-l  h30-h  h30-t 都是H30
            match = optimizedSearch(r'(\w*)[\s\-_]+[a-z0-9]+', uaData['device']['model'], re.I)
            if match:
                uaData['device']['model'] = match.group(1)

        elif getMatch(ua, r'(Build/HONOR|HW-CHM)', True):
            uaData['device']['manufacturer'] = 'Huawei'
            uaData['device']['model'] = 'Honor'

        # handle Xiaomi
        # 兼容build结尾或直接)结尾 以及特殊的HM处理方案(build/hm2013011)
        # xiaomi手机类型: mi m1 m2 m3 hm 开头
        # hongmi有特殊判断build/hm2015011

        # eg: Mozilla/5.0 (Linux; Android 11; M2004J19C Build/RP1A.200720.011; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/76.0.3809.89 Mobile Safari/537.36 T7/13.12 SP-engine/2.52.0 baiduboxapp/13.14.0.11 (Baidu; P1 11) NABar/1.0
        elif getMatch(ua, r';\s(mi|m[1-9]|hm)(\s+\w*)[\s\)]', True):
            match = getMatch()
            tmpMatch = optimizedSearch(r'(meitu|MediaPad)', ua, re.I)
            if tmpMatch:
                # 美图手机名字冒充小米 比如 meitu m4 mizhi
                uaData['device']['manufacturer'] = tmpMatch.group(1)
                uaData['device']['model'] = ''

            # 若匹配出的 match.group(2)没空格 会出现很多例如 mizi mizhi miha 但也会出现mi3 minote之类 特殊处理下
            elif len(match.group(2)) > 0 and not optimizedSearch(r'\s', match.group(2)):
                tmpMatch = optimizedSearch(r'(\d)', match.group(2), re.I)
                if tmpMatch:
                    uaData['device']['model'] = match.group(1) + '-' + tmpMatch.group(1)


            else:
                uaData['device']['manufacturer'] = 'Xiaomi'
                if match.group(2) and len(match.group(2)) > 0:
                    tmp = optimizedSub(r'\s', '', match.group(2), 1)
                    # tmp = match.group(2).replace(/\s/, '')
                    start = len(match.group(1)) - 2
                    uaData['device']['model'] = (match.group(1)[start:] + '-' + tmp)

                    uaData['device']['model'] = optimizedSub(r'm(\d)-', r'MI-\1', uaData['device']['model'], 1, re.I)
                    # uaData['device']['model'] = uaData['device']['model'].replace(//i, 'MI-$1')

                else:
                    start = len(match.group(1)) - 2
                    uaData['device']['model'] = match.group(1)[start:]
                    uaData['device']['model'] = optimizedSub(r'm(\d)', r'MI-\1', uaData['device']['model'], 1, re.I)

                # 解决移动联通等不同发行版导致的机型不同问题
                # 特征：mi-3c,例如mi-4LTE mi-4 其实应该是 mi-4
                if optimizedSearch(r'(mi|hm)(-\d)', uaData['device']['model'], re.I):
                    # 看看是不是 MI-3S  MI-4S....
                    match = optimizedSearch(r'(mi|hm)(-\ds)', uaData['device']['model'], re.I)
                    if match:
                        uaData['device']['model'] = match.group(1) + match.group(2)

                    # 防止 MI-20150XX等滥竽充数成为MI-2
                    elif getMatch(uaData['device']['model'], r'(mi|hm)(-\d{2})', True):
                        match = getMatch()
                        uaData['device']['model'] = match.group(1)

                    # 将mi-3c mi-3a mi-3w等合为mi-3
                    elif getMatch(uaData['device']['model'], r'(mi|hm)(-\d)[A-Z]', True):
                        match = getMatch()
                        uaData['device']['model'] = match.group(1) + match.group(2)


                # 去除 mi-4g这样的东西
                if getMatch(uaData['device']['model'], r'(mi|hm)(-\dg)', True):
                    match = getMatch()
                    uaData['device']['model'] = match.group(1)

        elif getMatch(ua, r'(MIX)', True):
            uaData['device']['manufacturer'] = 'Xiaomi'
            uaData['device']['model'] = 'MIX'

        elif getMatch(ua, r'build\/HM\d{0,7}\)', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Xiaomi'
            uaData['device']['model'] = 'HM'

        elif getMatch(ua, r'\sM\w{7,9}\s', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Xiaomi'
            uaData['device']['model'] = 'Xiaomi'

        elif getMatch(ua, r'redmi\s?(\d+)?', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Xiaomi'
            # fix: Mozilla/5.0 (Linux; Android 8.1.0; Redmi Note 5 Build/OPM1.171019.011; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/63.0.3239.83 Mobile Safari/537.36 T7/11.1 baiduboxapp/11.1.5.10 (Baidu; P1 8.1.0)
            # 比原有js多的逻辑，保证和js输出一致
            uaData['device']['model'] = 'HM-' + str(match.group(1) if match.group(1) else 'UNDEFINED')

        elif uaData['device']['manufacturer'] and uaData['device']['manufacturer'].lower() == 'xiaomi' and uaData['device']['model']:
            # 针对通过base库判断出数据时命名风格不同。特殊处理适配如下
            if getMatch(uaData['device']['model'], r'mi-one', True):
                uaData['device']['model'] = 'MI-1'

            # mi 2
            elif getMatch(uaData['device']['model'], r'mi-two', True):
                uaData['device']['model'] = 'MI-2'

            # 20150xxx2014501
            elif getMatch(uaData['device']['model'], r'\d{6}', True):
                uaData['device']['model'] = ''
            elif getMatch(uaData['device']['model'], r'redmi', True):
                uaData['device']['model'] = optimizedSub(r'redmi', r'HM', uaData['device']['model'].upper(), 1, re.I)
                # uaData['device']['model'] = uaData['device']['model'].upper().replace(/redmi/i, 'HM')

            # m1 m2 m3 写法不标准 另外判断是否是 m1-s
            elif getMatch(uaData['device']['model'], r'(m\d)[\s\-_](s?)', True):
                match = getMatch()
                uaData['device']['model'] = optimizedSub(r'm', r'MI-', match.group(1), 1, re.I) + match.group(2)
                # uaData['device']['model'] = match.group(1).replace(/m/, 'MI-') + match.group(2)

            # mi-2w  mi-3w 等格式化为mi-2  mi-3
            elif getMatch(uaData['device']['model'], r'(hm|mi)[\s\-_](\d?)[a-rt-z]', True):
                match = getMatch()
                tmpMatch = getMatch(uaData['device']['model'], r'(mi|hm)[\s\-_](note|pad)(\d?s?)', True)
                if tmpMatch:
                    uaData['device']['model'] = tmpMatch.group(1) + '-' + tmpMatch.group(2) + tmpMatch.group(3)

                else:
                    uaData['device']['model'] = (match.group(1) + '-' + match.group(2)) if match.group(2) else match.group(1)


            # 处理hm
            elif getMatch(uaData['device']['model'], r'hm', True):
                match = getMatch()
                # 判断是不是 hm-201xxx充数
                # if match = uaData['device']['model'].match(//i):
                if getMatch(uaData['device']['model'], r'(hm)[\s\-_](\d{2})', True):
                    match = getMatch()
                    uaData['device']['model'] = 'HM'

                # 判断是不是 hm-2s hm-1s
                # elif match = uaData['device']['model'].match(//i):
                elif getMatch(uaData['device']['model'], r'(hm)[\s\-_](\ds)', True):
                    match = getMatch()
                    uaData['device']['model'] = 'HM-' + match.group(2)

                # elif match = uaData['device']['model'].match(//i):
                elif getMatch(uaData['device']['model'], r'(hm)[\s\-_](\d)[a-z]', True):
                    match = getMatch()
                    uaData['device']['model'] = 'HM-' + match.group(2)

                else:
                    uaData['device']['model'] = 'HM'

                # 过滤类似 2g 3g等数据
                if optimizedSearch(r'hm-\dg', uaData['device']['model']):
                    uaData['device']['model'] = 'HM'



        # handle Vivo
        # 兼容build结尾或直接)结尾
        # vivo机型特征: Vivo[\s\-_](\w*)  或者以 E1  S11t  S7t 等开头
        # elif match = ua.match(//i):
        elif getMatch(ua, r'(vivo[\s\-_](\w*)|\s(E1\w?|E3\w?|E5\w?|V1\w?|V2\w?|S11\w?|S12\w?|S1\w?|S3\w?|S6\w?|S7\w?|S9\w?|X1\w?|X3\w?|X520\w?|X5\w?|X5Max|X5Max+|X5Pro|X5SL|X710F|X710L|Xplay|Xshot|Xpaly3S|Y11\w?|Y11i\w?|Y11i\w?|Y13\w?|Y15\w?|Y17\w?|Y18\w?|Y19\w?|Y1\w?|Y20\w?|Y22\w?|Y22i\w?|Y23\w?|Y27\w?|Y28\w?|Y29\w?|Y33\w?|Y37\w?|Y3\w?|Y613\w?|Y622\w?|Y627\w?|Y913\w?|Y923\w?|Y927\w?|Y928\w?|Y929\w?|Y937\w?|V19\w+|V18\w+)[\s\)])', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Vivo'
            uaData['device']['model'] = match.group(1)
            # 首先剔除 viv-  vivo-  bbg- 等打头的内容
            uaData['device']['model'] = optimizedSub(r'(viv[\s\-_]|vivo[\s\-_]|bbg[\s\-_])', r'', uaData['device']['model'], 1, re.I)
            # 解决移动联通等不同发行版导致的机型不同问题
            # 特征：[A-Z][0-9]+[A-Z] 例如  X5F X5L X5M X5iL 都应该是 X5
            if getMatch(uaData['device']['model'], r'([a-z]+[0-9]+)i?[a-z]?[\s\-_]?', True):
                match = getMatch()
                uaData['device']['model'] = match.group(1)
        # handle vivo
        # eg: Mozilla/5.0 (Linux; Android 10; V2065A Build/QP1A.190711.020; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/76.0.3809.89 Mobile Safari/537.36 T7/13.7 matrixstyle/1 SP-engine/2.43.0 bdapp/1.0 (tomas; tomas) tomas/1.34.0.14 (Baidu; P1 10) NABar/1.0
        elif getMatch(ua, r'(\sV([\w]{5,6})[\s\)])', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Vivo'
            uaData['device']['model'] = 'Vivo'
        elif getMatch(ua, r'(X[\d]{2});', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Vivo'
            uaData['device']['model'] = match.group(1)


        # handle Oppo
        # elif match = ua.match(//i):
        elif getMatch(ua, r'(Oppo[\s\-_](\w*)|\s(1100|1105|1107|3000|3005|3007|6607|A100|A103|A105|A105K|A109|A109K|A11|A113|A115|A115K|A121|A125|A127|A129|A201|A203|A209|A31|A31c|A31t|A31u|A51kc|A520|A613|A615|A617|E21W|Find|Mirror|N5110|N5117|N5207|N5209|R2010|R2017|R6007|R7005|R7007|R7c|R7t|R8000|R8007|R801|R805|R807|R809T|R8107|R8109|R811|R811W|R813T|R815T|R815W|R817|R819T|R8200|R8205|R8207|R821T|R823T|R827T|R830|R830S|R831S|R831T|R833T|R850|Real|T703|U2S|U521|U525|U529|U539|U701|U701T|U705T|U705W|X9000|X9007|X903|X905|X9070|X9077|X909|Z101|R829T)[\s\)])', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Oppo'
            if match.group(2):
                uaData['device']['model'] = match.group(2)

            elif match.group(3):
                uaData['device']['model'] = match.group(3)

            # 解决移动联通等不同发行版导致的机型不同问题
            # 特征：[A-Z][0-9]+[A-Z] 例如  A31c A31s 都应该是 A31
            # 对 Plus 做特殊处理
            if getMatch(uaData['device']['model'], r'([a-z]+[0-9]+)-?(plus)', True):
                match = getMatch()
                uaData['device']['model'] = match.group(1) + '-' + match.group(2)

            elif getMatch(uaData['device']['model'], r'(\w*-?[a-z]+[0-9]+)', True):
                match = getMatch()
                uaData['device']['model'] = match.group(1)
        # handle oppo
        # eg: Mozilla/5.0 (Linux; U; Android 11; zh-cn; PDVM00 Build/RKQ1.201217.002) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/90.0.4430.61 Mobile Safari/537.36 HeyTapBrowser/40.7.39.5
        elif getMatch(ua, r'(\sP([\w]{5})\s)', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Oppo'
            uaData['device']['model'] = 'Oppo'

        elif getMatch(ua, r'(Build/OPM|Build/O11019)', True):
            uaData['device']['manufacturer'] = 'Oppo'
            uaData['device']['model'] = 'A'

        elif uaData['device']['manufacturer'] and uaData['device']['manufacturer'].lower() == 'oppo' and uaData['device']['model']:
            # 针对base库的数据做数据格式化处理
            # 解决移动联通等不同发行版导致的机型不同问题
            # 特征：[A-Z][0-9]+[A-Z] 例如  A31c A31s 都应该是 A31
            # 对 Plus 做特殊处理
            # if match = uaData['device']['model'].match(//i):
            if getMatch(uaData['device']['model'], r'([a-z]+[0-9]+)-?(plus)', True):
                match = getMatch()
                uaData['device']['model'] = match.group(1) + '-' + match.group(2)

            elif getMatch(uaData['device']['model'], r'(\w*-?[a-z]+[0-9]+)', True):
                match = getMatch()
                uaData['device']['model'] = match.group(1)


        # handle Lenovo
        # 兼容build结尾或直接)结尾 兼容Lenovo-xxx/xxx以及Leveno xxx build等
        elif getMatch(ua, r'(Lenovo[\s\-_](\w*[-_]?\w*)|\s(A3580|A3860|A5500|A5600|A5860|A7600|A806|A800|A808T|A808T-I|A936|A938t|A788t|K30-E|K30-T|K30-W|K50-T3s|K50-T5|K80M|K910|K910e|K920|S90-e|S90-t|S90-u|S968T|X2-CU|X2-TO|Z90-3|Z90-7)[\s\)])', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Lenovo'
            if match.group(2):
                uaData['device']['model'] = match.group(2)

            elif match.group(3):
                uaData['device']['model'] = match.group(3)

            # 解决移动联通等不同发行版导致的机型不同问题
            # 特征：[A-Z][0-9]+[A-Z] 例如  A360t A360 都应该是 A360
            if getMatch(uaData['device']['model'], r'([a-z]+[0-9]+)', True):
                match = getMatch()
                uaData['device']['model'] = match.group(1)


        # handle coolpad
        elif getMatch(ua, r'(Coolpad[\s\-_](\w*)|\s(7295C|7298A|7620L|8908|8085|8970L|9190L|Y80D)[\s\)])', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Coolpad'
            if match.group(2):
                uaData['device']['model'] = match.group(2)

            elif match.group(3):
                uaData['device']['model'] = match.group(3)


            # 解决移动联通等不同发行版导致的机型不同问题
            # 特征：[A-Z][0-9]+[A-Z] 例如  8297-t01 8297-c01 8297w 都应该是 8297
            if getMatch(uaData['device']['model'], r'([a-z]?[0-9]+)', True):
                match = getMatch()
                uaData['device']['model'] = match.group(1)


        elif uaData['device']['manufacturer'] and uaData['device']['manufacturer'].lower() == 'coolpad' and uaData['device']['model']:
            # base 库适配
            # 解决移动联通等不同发行版导致的机型不同问题
            # 特征：[A-Z][0-9]+[A-Z] 例如  8297-t01 8297-c01 8297w 都应该是 8297
            if getMatch(uaData['device']['model'], r'([a-z]?[0-9]+)', True):
                match = getMatch()
                uaData['device']['model'] = match.group(1)


        # handle meizu
        elif getMatch(ua, r'\s(mx\d*\w*|mz-(\w*))\s(\w*)\s*\w*\s*build', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Meizu'
            tmpModel = match.group(2) if match.group(2) else match.group(1)
            if match.group(3):
                uaData['device']['model'] = tmpModel + '-' + match.group(3)

            else:
                uaData['device']['model'] = tmpModel + ''


        elif getMatch(ua, r'(M463C|M35\d)', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Meizu'
            uaData['device']['model'] = match.group(1)

        # handle htc
        elif getMatch(ua, r'(Htc[-_\s](\w*)|\s(601e|606w|608t|609d|610t|6160|619d|620G|626d|626s|626t|626w|709d|801e|802d|802t|802w|809D|816d|816e|816t|816v|816w|826d|826s|826t|826w|828w|901e|919d|A310e|A50AML|A510e|A620d|A620e|A620t|A810e|A9191|Aero|C620d|C620e|C620t|D316d|D516d|D516t|D516w|D820mt|D820mu|D820t|D820ts|D820u|D820us|E9pt|E9pw|E9sw|E9t|HD7S|M8Et|M8Sd|M8St|M8Sw|M8d|M8e|M8s|M8si|M8t|M8w|M9W|M9ew|Phablet|S510b|S510e|S610d|S710d|S710e|S720e|S720t|T327t|T328d|T328t|T328w|T329d|T329t|T329w|T528d|T528t|T528w|T8698|WF5w|X315e|X710e|X715e|X720d|X920e|Z560e|Z710e|Z710t|Z715e)[\s\)])', False):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Htc'
            uaData['device']['model'] = match.group(1)

        # handle Gionee
        elif getMatch(ua, r'(Gionee[\s\-_](\w*)|\s(GN\d+\w*)[\s\)])', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Gionee'
            if match.group(2):
                uaData['device']['model'] = match.group(2)

            elif match.group(3):
                uaData['device']['model'] = match.group(3)


        # handle LG
        elif getMatch(ua, r'(LG[-_](\w*)|\s(D728|D729|D802|D855|D856|D857|D858|D859|E985T|F100L|F460|H778|H818|H819|P895|VW820)[\s\)])', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Lg'
            if match.group(2):
                uaData['device']['model'] = match.group(2)

            elif match.group(3):
                uaData['device']['model'] = match.group(3)


        # handle tcl
        elif getMatch(ua, r'(Tcl[\s\-_](\w*)|\s(H916T|P588L|P618L|P620M|P728M)[\s\)])', False):
            match = getMatch()

            uaData['device']['manufacturer'] = 'Tcl'
            uaData['device']['model'] = match.group(1)

        # ZTE
        elif getMatch(ua, r'(V9180|N918)', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Zte'
            uaData['device']['model'] = match.group(1)

        elif uaData['device']['manufacturer'] and uaData['device']['manufacturer'].lower() == 'zte' and uaData['device']['model']:
            # base 库适配
            # 解决移动联通等不同发行版导致的机型不同问题
            # 特征：[A-Z][0-9]+[A-Z] 例如  Q505T Q505u 都应该是 Q505
            if getMatch(uaData['device']['model'], r'([a-z]?[0-9]+)', True):
                match = getMatch()
                uaData['device']['model'] = match.group(1)


        # UIMI
        elif getMatch(ua, r'(UIMI\w*|umi\w*)[\s\-_](\w*)\s*\w*\s*build', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Uimi'
            if match.group(2):
                uaData['device']['model'] = match.group(1) + '-' + match.group(2)

            else:
                uaData['device']['model'] = match.group(1) + ''


        # eton
        elif getMatch(ua, r'eton[\s\-_](\w*)', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Eton'
            uaData['device']['model'] = match.group(1)

        # Smartisan
        elif getMatch(ua, r'(SM705|SM701|YQ601|YQ603)', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Smartisan'
            tmpMap = {
                'SM705': 'T1',
                'SM701': 'T1',
                'YQ601': 'U1',
                'YQ603': 'U1'
            }
            if match.group(1) in tmpMap:
                uaData['device']['model'] = tmpMap[match.group(1)]
            else:
                uaData['device']['model'] = match.group(1)
            # uaData['device']['model'] = ({
            #     SM705: 'T1',
            #     SM701: 'T1',
            #     YQ601: 'U1',
            #     YQ603: 'U1'
            # })[]or match.group(1)

        # handle Asus
        elif getMatch(ua, r'(Asus[\s\-_](\w*)|\s(A500CG|A500KL|A501CG|A600CG|PF400CG|PF500KL|T001|X002|X003|ZC500TG|ZE550ML|ZE551ML)[\s\)])', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Asus'
            if match.group(2):
                uaData['device']['model'] = match.group(2)

            elif match.group(3):
                uaData['device']['model'] = match.group(3)


        # handle nubia
        elif getMatch(ua, r'(Nubia[-_\s](\w*)|(NX501|NX505J|NX506J|NX507J|NX503A|nx\d+\w*)[\s\)])', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Nubia'
            if match.group(2):
                uaData['device']['model'] = match.group(2)

            elif match.group(3):
                uaData['device']['model'] = match.group(3)


        # handle haier
        elif getMatch(ua, r'(HT-\w*)|Haier[\s\-_](\w*-?\w*)', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Haier'
            if match.group(1):
                uaData['device']['model'] = match.group(1)

            elif match.group(2):
                uaData['device']['model'] = match.group(2)


        # tianyu
        elif getMatch(ua, r'K-Touch[\s\-_](tou\s?ch\s?(\d)|\w*)', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'K-Touch'
            if match.group(2):
                uaData['device']['model'] = 'Ktouch' + match.group(2)

            else:
                uaData['device']['model'] = match.group(1)



        # DOOV
        elif getMatch(ua, r'Doov[\s\-_](\w*)', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Doov'
            uaData['device']['model'] = match.group(1)

        # coobee
        elif getMatch(ua, r'koobee', True):
            uaData['device']['manufacturer'] = 'koobee'


        # sony
        elif getMatch(ua, r'C69', True):
            uaData['device']['manufacturer'] = 'Sony'


        # haojixing
        elif getMatch(ua, r'N787|N818S', True):
            uaData['device']['manufacturer'] = 'Haojixing'


        # haisense
        elif getMatch(ua, r'(hs-|Hisense[\s\-_])(\w*)', True):
            match = getMatch()
            uaData['device']['manufacturer'] = 'Hisense'
            uaData['device']['model'] = match.group(2)

        #############################################################
        ###   格式化厂商
        ##########################################################

        # format the style of manufacturer
        if uaData['device']['manufacturer']:
            uaData['device']['manufacturer'] = uaData['device']['manufacturer'][0:1].upper() + uaData['device']['manufacturer'][1:].lower()

        # format the style of model
        if uaData['device']['model']:
            uaData['device']['model'] = optimizedSub(r'-+|_+|\s+', r' ', uaData['device']['model'].upper())
            uaData['device']['model'] = optimizedSearch(r'\s*(\w*\s*\w*)', uaData['device']['model']).group(1)
            uaData['device']['model'] = optimizedSub(r'\s+', r'-', uaData['device']['model'])

            # uaData['device']['model'] = uaData['device']['model'].upper().replace(//g, ' ')
            # uaData['device']['model'] = uaData['device']['model'].match(//)[1].replace(/\s+/, '-')

            # 针对三星、华为做去重的特殊处理
            if uaData['device']['manufacturer'] == 'Samsung':
                tmpMap = {
                    'SCH-I95': 'GT-I950',
                    'SCH-I93': 'GT-I930',
                    'SCH-I86': 'GT-I855',
                    'SCH-N71': 'GT-N710',
                    'SCH-I73': 'GT-S789',
                    'SCH-P70': 'GT-I915'
                }
                if uaData['device']['model'] in tmpMap:
                    uaData['device']['model'] = tmpMap[uaData['device']['model']]
                else:
                    uaData['device']['model'] = uaData['device']['model']

                # uaData['device']['model'] = ({
                #     'SCH-I95': 'GT-I950',
                #     'SCH-I93': 'GT-I930',
                #     'SCH-I86': 'GT-I855',
                #     'SCH-N71': 'GT-N710',
                #     'SCH-I73': 'GT-S789',
                #     'SCH-P70': 'GT-I915'
                # })[uaData['device']['model']]or uaData['device']['model']

            elif uaData['device']['manufacturer'] == 'Huawei':
                tmpMap = {
                    'CHE1': 'CHE',
                    'CHE2': 'CHE',
                    'G620S': 'G621',
                    'C8817D': 'G621'
                }
                if uaData['device']['model'] in tmpMap:
                    uaData['device']['model'] = tmpMap[uaData['device']['model']]
                else:
                    uaData['device']['model'] = uaData['device']['model']

                # uaData['device']['model'] = ({
                #     CHE1: 'CHE',
                #     CHE2: 'CHE',
                #     G620S: 'G621',
                #     C8817D: 'G621'
                # })[uaData['device']['model']]or uaData['device']['model']



        # 针对xiaomi 的部分数据没有格式化成功，格式化1次
        if uaData['device']['manufacturer'] and uaData['device']['manufacturer'] == 'Xiaomi':
            if getMatch(uaData['device']['model'], r'(hm|mi)-(note)', True):
                match = getMatch()
                uaData['device']['model'] = match.group(1) + '-' + match.group(2)

            elif getMatch(uaData['device']['model'], r'(hm|mi)-(\ds?)', True):
                match = getMatch()
                uaData['device']['model'] = match.group(1) + '-' + match.group(2)

            elif getMatch(uaData['device']['model'], r'(hm|mi)-(\d)[a-rt-z]', True):
                match = getMatch()
                uaData['device']['model'] = match.group(1) + '-' + match.group(2)



    # handle browser
    # if (!uaData['browser']['name']) {
    # ua = ua.toLowerCase();
    if uaData['device']['type'] == 'desktop':

        # 360 security Explorer

        # user-agent: Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.5359.95 Safari/537.36 QIHU 360SE/14.1.1094.0
        if getMatch(ua, r'360se(?:[ \/]([\w.]+))?', True):
            match = getMatch()
            uaData['browser']['name'] = '360 security Explorer'
            uaData['browser']['version'] = {
                'original': match.group(1)
            }

        # the world

        # elif match = //i.exec(ua):
        elif getMatch(ua, r'the world(?:[ \/]([\w.]+))?', True):
            match = getMatch()
            uaData['browser']['name'] = 'the world'
            uaData['browser']['version'] = {
                'original': match.group(1)
            }


        # tencenttraveler

        # elif match = //i.exec(ua):
        elif getMatch(ua, r'tencenttraveler ([\w.]+)', True):
            match = getMatch()
            uaData['browser']['name'] = 'tencenttraveler'
            uaData['browser']['version'] = {
                'original': match.group(1)
            }


        # LBBROWSER

        # elif match = /LBBROWSER/i.exec(ua):
        elif getMatch(ua, r'LBBROWSER', True):
            match = getMatch()
            uaData['browser']['name'] = 'LBBROWSER'


    elif uaData['device']['type'] == 'mobile' or uaData['device']['type'] == 'tablet':

        # BaiduHD
        if getMatch(ua, r'BaiduHD\s+([\w.]+)', True):
            match = getMatch()
            uaData['browser']['name'] = 'BaiduHD'
            uaData['browser']['version'] = {
                'original': match.group(1)
            }


        # 360 Browser
        elif getMatch(ua, r'360.s*aphone\s*browser\s*\(version\s*([\w.]+)\)', True):
            match = getMatch()
            uaData['browser']['name'] = '360 Mobile Browser'
            uaData['browser']['version'] = {
                'original': match.group(1)
            }


        # LieBaoFast
        # elif match = //i.exec(ua):
        elif getMatch(ua, r'LieBaoFast\/([\w.]+)', True):
            match = getMatch()
            uaData['browser']['name'] = 'LieBao Fast'
            uaData['browser']['version'] = {
                'original': match.group(1)
            }


        # LieBao
        elif getMatch(ua, r'LieBao\/([\w.]+)', True):
            match = getMatch()
            uaData['browser']['name'] = 'LieBao'
            uaData['browser']['version'] = {
                'original': match.group(1)
            }


        # SOUGOU
        elif getMatch(ua, r'Sogou\w+\/([0-9\.]+)', True):
            match = getMatch()
            uaData['browser']['name'] = 'SogouMobileBrowser'
            uaData['browser']['version'] = {
                'original': match.group(1)
            }

        # 百度国际
        elif getMatch(ua, r'bdbrowser\w+\/([0-9\.]+)', True):
            match = getMatch()
            uaData['browser']['name'] = '百度国际'
            uaData['browser']['version'] = {
                'original': match.group(1)
            }

        # Android Chrome Browser
        elif uaData['os']['name'] == 'Android' and optimizedSearch(r'safari', ua, re.I) and getMatch(ua, r'chrome\/([0-9\.]+)', True):
            match = getMatch()
            tmpMatch = getMatch(ua, r'\s+(\w+Browser)\/?([\d\.]*)')
            if tmpMatch:
                uaData['browser']['name'] = tmpMatch.group(1)
                if tmpMatch.group(2):
                    uaData['browser']['version'] = {'original': tmpMatch.group(2)}
                else:
                    uaData['browser']['version'] = {'original': match.group(1)}

            else:
                uaData['browser']['name'] = 'Android Chrome'
                uaData['browser']['version'] = {'original': match.group(1)}


        # Android Google Browser
        elif uaData['os']['name'] == 'Android' and optimizedSearch(r'safari', ua, re.I) and getMatch(ua, r'version\/([0-9\.]+)', True):
            match = getMatch()
            tmpMatch = getMatch(ua, r'\s+(\w+Browser)\/?([\d\.]*)')
            if tmpMatch:
                uaData['browser']['name'] = tmpMatch.group(1)
                if tmpMatch.group(2):
                    uaData['browser']['version'] = {'original': tmpMatch.group(2)}
                else:
                    uaData['browser']['version'] = {'original': match.group(1)}

            else:
                uaData['browser']['name'] = 'Android Browser'
                uaData['browser']['version'] = {'original': match.group(1)}


        # 'Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206' belongs to Safari
        elif getMatch(ua, r'(ipad|iphone).* applewebkit\/.* mobile', True):
            match = getMatch()
            uaData['browser']['name'] = 'Safari'


    if getMatch(ua, r'baiduboxapp\/?([\d\.]*)', True):
        match = getMatch()
        uaData['browser']['name'] = 'BaiduApp'
        if match.group(1):
            uaData['browser']['version'] = {
                'original': match.group(1)
            }
    elif getMatch(ua, r'BaiduLightAppRuntime', True):
        match = getMatch()
        uaData['browser']['name'] = '轻应用runtime'
    elif getMatch(ua, r'baidubrowser\/([\d\.]+)\s', True):
        match = getMatch()
        uaData['browser']['name'] = 'BaiduBrowser'
        uaData['browser']['version'] = {
            'original': match.group(1)
        }
    elif getMatch(ua, r'SE 2.X MetaSr', True):
        # match = getMatch()
        uaData['browser']['name'] = 'SogouExplorer'
        uaData['browser']['version'] = {
            'original': ''
        }
    elif getMatch(ua, r'PhantomJS\/([0-9.]*)', True):
        match = getMatch()
        uaData['browser']['name'] = 'PhantomJS'
        uaData['browser']['version'] = {
            'original': match.group(1)
        }
    elif getMatch(ua, r'Weibo', True):
        match = getMatch()
        uaData['browser']['name'] = '微博'
    elif getMatch(ua, r'MQQ', True):
        match = getMatch()
        uaData['browser']['name'] = '手机QQ'
    elif getMatch(ua, r'hao123', True):
        match = getMatch()
        uaData['browser']['name'] = 'hao123'


    if getMatch(ua, r'MicroMessenger\/([\w.]+)', True):
        match = getMatch()
        uaData['browser']['name'] = '微信'
        tmpVersion = match.group(1).replace('_', '.')
        tmpMatch = getMatch(tmpVersion, r'(\d+\.\d+\.\d+\.\d+)')
        if tmpMatch:
            tmpVersion = tmpMatch.group(1)

        uaData['browser']['version'] = {
            'original': tmpVersion
        }

    elif getMatch(ua, r'UCBrowser\/([\w.]+)', True):
        match = getMatch()
        uaData['browser']['name'] = 'UC Browser'
        uaData['browser']['version'] = {
            'original': match.group(1)
        }

    # Lenovo
    # Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.131 Safari/537.36 SLBrowser/8.0.1.5162 SLBChan/10
    elif getMatch(ua, r'SLBrowser\/([\w.]+)', True):
        match = getMatch()
        uaData['browser']['name'] = 'Lenovo'
        uaData['browser']['version'] = {
            'original': match.group(1)
        }

    if getMatch(ua, r'OPR\/([\w.]+)', True):
        match = getMatch()
        uaData['browser']['name'] = 'Opera'
        uaData['browser']['version'] = {
            'original': match.group(1)
        }
    elif getMatch(ua, r'OPiOS\/([\w.]+)', True):
        match = getMatch()
        uaData['browser']['name'] = 'Opera'
        uaData['browser']['version'] = {
            'original': match.group(1)
        }

    # IE 11
    # elif //i.test(ua) and //i.test(ua):
    elif getMatch(ua, r'Trident\/7', True) and getMatch(ua, r'rv:11', True):
        uaData['browser']['name'] = 'Internet Explorer'
        uaData['browser']['version'] = {
            'major': '11',
            'original': '11'
        }

    # Microsoft Edge
    # Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36 Edg/111.0.1661.62
    elif getMatch(ua, r' (Edge|Edg)\/([\w.]+)', True):
        match = getMatch()
        uaData['browser']['name'] = 'Microsoft Edge'
        uaData['browser']['version'] = {
            'original': match.group(2)
        }

    # miui browser
    # elif match = //i.exec(ua):
    elif getMatch(ua, r'miuibrowser\/([\w.]+)', True):
        match = getMatch()
        uaData['browser']['name'] = 'miui browser'
        uaData['browser']['version'] = {
            'original': match.group(1)
        }

    # 修正部分厂商
    # Dalvik/2.1.0 (Linux; U; Android 8.1.0; M1813 Build/O11019)
    if getMatch(ua, r'(M1813)', True):
        uaData['device']['manufacturer'] = 'Meizu'
        uaData['device']['model'] = ''


    # Safari
    if not uaData['browser']['name']:
        if getMatch(ua, r'Safari\/([\w.]+)', True) and optimizedSearch(r'Version', ua, re.I):
            match = getMatch()
            uaData['browser']['name'] = 'Safari'


    if uaData['browser']['name'] and not uaData['browser']['version']['original']:
        # if match = //i.exec(ua):
        if getMatch(ua, r'Version\/([\w.]+)', True):
            match = getMatch()
            uaData['browser']['version'] =  {
                'original': match.group(1)
            }



    # if (uaData['os']['name'] === 'Windows' && uaData['os']['version']) {
    #  // Windows 8.1
    #  if (uaData['os']['version'].alias === 'NT 6.3') {
    #      uaData['os']['version'].alias = '8.1';
    #  }
    # }
    # handle os
    if uaData['os']['name'] == 'Windows'or optimizedSearch(r'Windows', ua, re.I):
        uaData['os']['name'] = 'Windows'
        if optimizedSearch(r'NT 6.3', ua, re.I):
            uaData['os']['version'] = {
                'alias': '8.1',
                'original': '8.1'
            }

        elif optimizedSearch(r'NT 6.4', ua, re.I) or optimizedSearch(r'NT 10.0', ua, re.I):
            uaData['os']['version'] = {
                'alias': '10',
                'original': '10'
            }


    elif uaData['os']['name'] == 'Mac OS X':
        uaData['os']['name'] = 'Mac OS X'
        if getMatch(ua, r'Mac OS X[\s\_\-\/](\d+[\.\-\_]\d+[\.\-\_]?\d*)', True):
            match = getMatch()
            uaData['os']['version'] = {
                'alias': match.group(1).replace('_', '.'),
                'original': match.group(1).replace('_', '.')
            }

        else:
            uaData['os']['version'] = {
                'alias': '',
                'original': ''
            }


    elif optimizedSearch(r'Android', uaData['os']['name'], re.I):
        if getMatch(ua, r'Android[\s\_\-\/i686]?[\s\_\-\/](\d+[\.\-\_]\d+[\.\-\_]?\d*)', True):
            match = getMatch()
            uaData['os']['version'] = {
                'alias': match.group(1),
                'original': match.group(1)
            }


    return uaData






#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: kaivean
# Mail: kaivean@outlook.com
# 1: modify following version
# 2. build and upload
#   usage test: rm -fr dist && python3 setup.py sdist  && twine upload --repository-url https://test.pypi.org/legacy/ dist/uaDevice-*.tar.gz
#   usage: rm -fr dist && python3 setup.py sdist  && twine upload dist/uaDevice-*.tar.gz
#############################################

import os
from setuptools import setup, find_packages
ROOT = os.path.dirname(os.path.realpath(__file__))

import sys
import re
import os
import json

if sys.version_info < (3, 0):
    desc = open(os.path.join(ROOT, 'README.md')).read()
else:
    desc = open(os.path.join(ROOT, 'README.md'), encoding='UTF-8').read()

setup(
    name = "uaDevice",
    version = "1.0.10",
    keywords = ("ua", "user-agent", "User Agent", "parser", "device", "os", "browser", "engine", "data analysis", "china", "中国", "国内"),
    description = "User Agent parser, More accurate",
    long_description = desc,
    long_description_content_type="text/markdown",
    license = "MIT Licence",

    url = "https://github.com/kaivean/python-ua-device",
    author = "kaivean",
    author_email = "kaivean@outlook.com",

    packages = find_packages(exclude=['*.test', '*.test.*', 'test.*', 'test', 'test.py']),
    include_package_data = True,
    platforms = "any",
    install_requires = []
)
